from __future__ import print_function
#
#   Obsolete module (use connection module)
#
import subprocess
import wx
import os
import shlex
import socket

module_name = 'SSH'
class_name = 'SSH'
menu = [("Connect",    "onOpen",    False),
        ("Disconnect", "onClose",  False),
        ("Send",       "onSend",  False),
        ("Recieve",    "onRec",  False),
        ("Exec",       "onExec",  False),
        ("Setting...", "onSetting",  False)]
method = ['clean', 'setSetting', 'onSetting',
          'onOpen', 'onClose', 'onSend',
          'onRec', 'onExec']
icon = 'world_link.png'
can_have_child = False
has_private_owndir = False


def scp_paramiko(localhost, port, lfile, rfile):

    import paramiko
#  username = ('user')
#  password = ('1234')
    hostname = localhost
    ports = port
    localD = (lfile)
    remoteD = (rfile)

    paramiko.util.log_to_file('/tmp/paramiko.log')
    transport = paramiko.Transport(('localhost', 10001))
    transport.connect()
    sftp = paramiko.SFTPClient.from_transport(transport)
    sftp.put(remotepath=remoteD, localpath=localD)

    sftp.close()
    transport.close()


def pick_unused_port():
    '''
    a routeint to le a system to pick an unused port.
    this recipe was found in active state web site
    '''
    import socket
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind(('localhost', 0))
    addr, port = s.getsockname()
    s.close()
    return port


def set_td_var(td, server=None, port=None,
               rport=None, mode=None,
               rserver=None):
    if server is not None:
        td.setvar("server", str(server))
    if port is not None:
        td.setvar("port", int(port))
    if rport is not None:
        td.setvar("rport", int(rport))
    if mode is not None:
        td.setvar("mode", str(mode))
    if rserver is not None:
        td.setvar("rserver", str(rserver))


def get_td_var(td):
    server = td.getvar("server")
    port = td.getvar("port")
    rport = td.getvar("rport")
    mode = td.getvar("mode")
    rserver = td.getvar("rserver")
    return server, port, rport, mode, rserver


def clean(self):
    # clean is a special process called when td is deleted
    if hasattr(self, 'p'):
        self.p.terminate()
        del self.p


def setSetting(self, value):
    rserver = value[4]
    if value[1] == 'loki':
        rserver = 'loki.psfc.mit.edu'

    set_td_var(self.td, server=str(value[4]),
               port=str(value[2]),
               rport=str(value[3]),
               mode=str(value[1]),
               rserver=rserver)
    self.td._name = str(value[1])
    self.td._status = 'not connected'

    app = self.td.get_root_parent().app
    app.proj_tree_viewer.update_widget()


def onRec(self, file=None, rfile=None, rdir=None):
    if rfile is None:
        print("please specify remote file")
        return
    if rdir is not None:
        rfile = os.path.join(rdir, rfile)
    m = self

    if file == None:
        file = os.path.join(os.getcwd(),
                            str(os.path.basename(rfile)))

    server, port, rport, mode, rserver = get_td_var(self.td)
    command = 'scp -P '+str(port) + ' localhost:'+rfile + ' '+file
    print(command)
    p = subprocess.Popen(shlex.split(command), stderr=subprocess.STDOUT,
                         stdout=subprocess.PIPE)
    p.wait()
    print(('stdout', p.stdout.read()))


def onExec(self, e=None, command=None, wait=True):
    if command is None:
        return
    server, port, rport, mode, rserver = get_td_var(self.td)

    command = 'ssh -p '+str(port) + ' localhost ' + command

    p = subprocess.Popen(shlex.split(command), stderr=subprocess.STDOUT,
                         stdout=subprocess.PIPE)

    if wait:
        while p.wait() == None:
            wx.SafeYield()
            time.sleep(0.25)
    print(('stdout', p.stdout.read()))


def onSend(self, file=None, rdir=None, rfile=None):
    if self.td._status != 'connected':
        print("SSH conection is not established")
        return

    if file is None:
        open_dlg = wx.FileDialog(None,
                                 message="Select file to send",
                                 style=wx.FD_SAVE)
        if open_dlg.ShowModal() != wx.ID_OK:
            open_dlg.Destroy()
            return
        file = open_dlg.GetPath()
        open_dlg.Destroy()

    rfile = str(os.path.basename(file))
    rdir = str(rdir)
    if rdir is not None:
        rfile = os.path.join(rdir, rfile)

    server, port, rport, mode, rserver = get_td_var(self.td)

    command = 'ssh -p '+str(port) + ' localhost mkdir ' + rdir

    p = subprocess.Popen(shlex.split(command), stderr=subprocess.STDOUT,
                         stdout=subprocess.PIPE)

    p.wait()
    print(('stdout', p.stdout.read()))

    #scp_paramiko('localhost', port, file, rfile)

    command = 'scp -P '+str(port) + ' '+str(file) + ' localhost:'+rfile
    print(command)
    subprocess.Popen(command, shell=True)
    print(p.wait())


def onClose(self):
    if hasattr(self, 'p'):
        self.p.terminate()
        del self.p
    self.td._name = self.td.getvar("mode")
    self.td._status = 'not connected'
    app = self.td.get_root_parent().app
    app.proj_tree_viewer.update_widget()


def onOpen(self):
    import subprocess
    import shlex
    import wx
    if hasattr(self, 'p'):
        self.p.terminate()
        del self.p

    m = self
    server, port, rport, mode, rserver = get_td_var(self.td)

    command = 'ssh -N '+str(server) + ' -L ' +  \
        str(port)+':' + str(rserver)+':' + \
        str(rport)
    print(command)
    self.p = subprocess.Popen(shlex.split(command))
    self.td._name = self.td.getvar("mode")
    self.td._status = 'connected'
    app = self.td.get_root_parent().app
    app.proj_tree_viewer.update_widget()


def onSetting(self):
    server, port, rport, mode, rserver = get_td_var(self.td)

    if port is None:
        port = str(10000)
    if server is None:
        server = 'cmodws60.psfc.mit.edu'
    if rport is None:
        rport = '22'
    if mode is None:
        mode = 'loki'
    list = [["", "SSH connection", 2],
            ["SSH connection", mode, 4,
             {"choices": ["loki", "MDSPlus_CMOD", "pyro_ns"]}],
            ["local port", str(port), 0],
            ["remote port", str(rport), 0],
            ["sever", server, 0]]

    app = self.td.get_root_parent().app
    app.proj_tree_viewer.OpenPanel(list, self, 'setSetting')



from __future__ import print_function
import ifigure
import os
import sys
import time
import subprocess
import random
from ifigure.utils.pickled_pipe import PickledPipe


#
#  debug setting
#
import ifigure.utils.debug as debug
debug.debug_default_level = 1
dprint1, dprint2, dprint3 = debug.init_dprints('MDSSPWokerPool')


class MDSSPWorkerPool(object):
    def __init__(self, num, *args):
        self.num = num
        self.pch = [None]*num
        dirname = os.path.dirname(ifigure.__file__)

        type = args[0]
        if len(args) > 1:
            host = args[1]
            port = args[2]
        else:
            host = None
            port = None

        file = 'mds_sp_'+type+'worker.py'
        args = []
        if type == 'proxy':
            args = [str(host), str(port)]
#        if type == 'direct': file = 'mds_sp_directworker.py'
#        if type == 'proxy':  file = 'mds_sp_proxyworker.py'
#        if type == 'echoback':  file = 'mds_sp_echobackworker.py'
        script = os.path.join(dirname, 'mdsplus', file)
        for i in range(num):
            #           print args
            p = subprocess.Popen(['python', script] + args,
                                 bufsize=-1,
                                 stdin=subprocess.PIPE,
                                 stdout=subprocess.PIPE)
            ch = PickledPipe(p.stdout, p.stdin)
            self.pch[i] = (p, ch)
        self.analysis = [None]*num
        self.task_count = [0]*num

    def set_analysis(self, ana, w_id):
        self.analysis[w_id] = ana

    def submit_batchjob(self, w_id, batch, names):
        k = 0
        p, ch = self.pch[w_id]
        ch.send((batch, names))
        self.task_count[w_id] = 1
        return w_id

    def check_batchfinished(self, w_id):
        if self.task_count[w_id] == 0:
            return 1, None
        p, ch = self.pch[w_id]
        if p.poll() is not None:
            self.task_count[w_id] = 0
            print('process died?')
            return -1, None
        data = ch.recv()
        if data is not None:
            self.task_count[w_id] = 0
            if data == 'error':
                #               self.task_count[w_id] = 0
                return -1, data
#           if self.task_count[w_id] == 0:
            return 1, data
        return 0, None

    def is_alive(self, w_id):
        p, ch = self.pch[w_id]
        return p.poll() is None

    def is_any_alive(self):
        return any([p.poll() is None for p, ch in self.pch])

    def terminate_all(self):
        for p, ch in self.pch:
            try:
                ch.send(-1)
                data = ch.recv(nowait=False)
            except:
                pass
            p.kill()
            p.wait()

    def get_worker(self):
        #  return a worker which is free.
        ans_list = []
        for i,  pch in enumerate(self.pch):
            p, ch = pch
            if (self.analysis[i] is None and
                    p.poll() is None):
                ans_list.append((p, i))
        if len(ans_list) == 0:
            return None, -1
        ret = ans_list[int(random.random()*len(ans_list))]
        return ret[0], ret[1]

    def get_num_worker(self):
        return len(self.pch)

    def get_pool_ready(self):
        ans_list = []
        for i,  pch in enumerate(self.pch):
            p, ch = pch
            if (self.analysis[i] is None and
                    p.poll() is None):
                ans_list.append((p, i))
        return (len(ans_list) == len(self.pch))




__author__ = "Syun'ichi Shiraiwa"
__copyright__ = "Copyright, S. Shiraiwa, PiScope Project"
__credits__ = ["Syun'ichi Shiraiwa"]
__version__ = "0.5"
__maintainer__ = "S. Shiraiwa"
__email__ = "shiraiwa@psfc.mit.edu"
__status__ = "beta"


from ifigure.mto.fig_obj import FigObj
from ifigure.mto.fig_page import FigPage
from ifigure.widgets.canvas.file_structure import *
import ifigure
import os
import ifigure.utils.cbook as cbook
import ifigure.utils.geom as geom
import ifigure.widgets.canvas.custom_picker as cpicker
import numpy as np
import weakref
import logging
import matplotlib
from matplotlib.patches import PathPatch
import matplotlib.path as mpath

from ifigure.widgets.undo_redo_history import GlobalHistory
from ifigure.widgets.undo_redo_history import UndoRedoArtistProperty
from ifigure.widgets.undo_redo_history import UndoRedoFigobjProperty
from ifigure.widgets.undo_redo_history import UndoRedoFigobjMethod

from ifigure.mto.generic_points import GenericPoint, GenericPointsHolder
from ifigure.mto.figobj_gpholder import FigObjGPHolder


class FigCurve(FigObjGPHolder):
    def __init__(self, path=None, width=1.0, trans1=['figure']*2,
                 figaxes1=None,  **kywds):
        '''
        Curve object:
           figaxe1, figaxes2: None or full_path string
        '''

        self._objs = []  # for debug....
        if "draggable" in kywds:
            self.setvar("draggable", kywds["draggable"])
            del kywds["draggable"]
        else:
            self.setvar("draggable", True)
        args = ()
        GenericPointsHolder.__init__(self, num=1)
        super(FigCurve, self).__init__(*args, **kywds)
        self.setvar("curve_path", path)
        self.setvar("trans1", trans1)
        arrow1 = kywds.pop('arrow1', [False, 'simple'])
        arrow2 = kywds.pop('arrow2', [False, 'simple'])
        self.setvar("arrow1", arrow1)
        self.setvar("arrow2", arrow2)

        if figaxes1 is not None:
            self.setvar("figaxes1", figaxes1.get_full_path())
        else:
            self.setvar("figaxes1", None)
        self.setvar("width", width)
        self.setp('use_var', True)

        self._cb_added = False
        self._drag_mode = 0
        self._drag_artist = None
        self._hit_seg_i = -1

    @classmethod
    def isFigCurve(self):
        return True

    @classmethod
    def can_have_child(self, child=None):
        return False

    @classmethod
    def get_namebase(self):
        return 'curve'

    @classmethod
    def property_in_file(self):
        return ["facecolor", "edgecolor", "zorder", "alpha", "linewidth",
                "linestyle", "fill"]

    @classmethod  # define _attr values to be saved
    def attr_in_file(self):
        return (["curve_path"] +
                super(FigCurve, self).attr_in_file())

    @classmethod
    def property_in_palette(self):
        return ["path", "patch", "arrow"], [["edgecolor", "linewidth",
                                             "plinestyle", "alpha", "closepoly"],
                                            ["fill", "facecolor"],
                                            ["curvearrow1", "curvearrow2"]]

    @classmethod
    def load_classimage(self):
        from ifigure.ifigure_config import icondir as path
        idx1 = cbook.LoadImageFile(path, 'curve.png')
        return [idx1]

    def get_containter(self):
        return self.get_figpage()._artists[0]

#    def set_parent(self, parent):
#        FigObj.set_parent(self, parent)
#        GenericPointsHolder.set_gp_figpage(self)

    def canvas_menu(self):
        if self._drag_mode == 2:
            p = self.get_path()
            if self._drag_hit == -1:
                m = [("Add node",  self.onAddNode, None), ]
                if self._hit_seg_i != -1:
                    segpath = cbook.BezierSplit(self.get_path())
                    if len(segpath[self._hit_seg_i]) == 2:
                        m = m + \
                            [("Convert to curve",  self.onConvCurve, None), ]
                    else:
                        m = m + [("Convert to Line",  self.onConvLine, None), ]
                return m
            else:
                if cbook.BezierNodeType(p, self._drag_hit) == 1:
                    return [("Remove Node", self.onRmNode, None),
                            ("Convert to symmetric node", self.onMakeSym, None)]
                if cbook.BezierNodeType(p, self._drag_hit) == 2:
                    if p[self._drag_hit][2] == 1:  # smooth node
                        return [("Remove node", self.onRmNode, None),
                                ("Convert to coner node", self.onMakeCorner, None),
                                ("Convert to asymmetric node", self.onMakeAsym, None)]
                    elif p[self._drag_hit][2] == 2:  # coner node
                        return [("Remove Node", self.onRmNode, None),
                                ("Convert to symmetric node", self.onMakeSym, None)]
                    elif p[self._drag_hit][2] == 3:  # asymmetricr node
                        return [("Remove Node", self.onRmNode, None),
                                ("Convert to symmetric node", self.onMakeSym, None)]

        return []

    def onConvLine(self, evt):
        segpath = cbook.BezierSplit(self.get_path())
        p = segpath[self._hit_seg_i]
        segpath[self._hit_seg_i] = [p[0],
                                    (mpath.Path.LINETO, p[-1][1], 0)]
        path = cbook.BezierJoin(segpath)

        window = evt.GetEventObject().GetTopLevelParent()
        hist = GlobalHistory().get_history(window)
#        hist = self.get_root_parent().app.history
        hist.start_record()
        hist.add_history(UndoRedoFigobjMethod(self._artists[0],
                                              'pathdata', path))
        hist.stop_record()

    def onConvCurve(self, evt):
        segpath = cbook.BezierSplit(self.get_path())
        p = segpath[self._hit_seg_i]
        x0, y0 = p[0][1]
        x1, y1 = p[-1][1]
        segpath[self._hit_seg_i] = [p[0],
                                    (mpath.Path.CURVE4,
                                     (x0*0.7+x1*0.3, y0*0.7+y1*0.3), p[0][2]),
                                    (mpath.Path.CURVE4,
                                     (x0*0.3+x1*0.7, y0*0.3+y1*0.7), 2),
                                    (mpath.Path.CURVE4, p[-1][1], 2)]
        path = cbook.BezierJoin(segpath)

        window = evt.GetEventObject().GetTopLevelParent()
        hist = GlobalHistory().get_history(window)
        hist.start_record()
        hist.add_history(UndoRedoFigobjMethod(self._artists[0],
                                              'pathdata', path))
        hist.stop_record()

    def onAddNode(self, evt):
        fig_page = self.get_figpage()
        xy = self._artists[0].get_verts()
        path = self.get_path()
        hit, idx = cbook.BezierHitTest(path, self._st_p[0], self._st_p[1])
        if hit:
            path = cbook.BezierInsert(path, idx, self._st_p[0], self._st_p[1])
            window = evt.GetEventObject().GetTopLevelParent()
            hist = GlobalHistory().get_history(window)
#            hist = self.get_root_parent().app.history
            hist.start_record()
            hist.add_history(UndoRedoFigobjMethod(self._artists[0],
                                                  'pathdata', path))
            hist.stop_record()

    def onRmNode(self, evt):
        p = self.get_path()
        p = cbook.BezierRmnode(p, self._drag_hit)
        if p is None:
            pass

        window = evt.GetEventObject().GetTopLevelParent()
        hist = GlobalHistory().get_history(window)
 #       hist = self.get_root_parent().app.history
        hist.start_record()
        hist.add_history(UndoRedoFigobjMethod(self._artists[0],
                                              'pathdata', p))
        hist.stop_record()

    def onMakeCorner(self, evt):
        p = self.get_path()
        p = [p[i] for i in range(len(p))]
        p[self._drag_hit] = (p[self._drag_hit][0], p[self._drag_hit][1], 2)
        p[self._drag_hit-1] = (p[self._drag_hit-1][0],
                               p[self._drag_hit-1][1], 2)
        p[self._drag_hit+1] = (p[self._drag_hit+1][0],
                               p[self._drag_hit+1][1], 2)

        window = evt.GetEventObject().GetTopLevelParent()
        hist = GlobalHistory().get_history(window)
#        hist = self.get_root_parent().app.history
        hist.start_record()
        hist.add_history(UndoRedoFigobjMethod(self._artists[0],
                                              'pathdata', p))
        hist.stop_record()

    def onMakeSym(self, evt):
        p = self.get_path()
        p = [p[i] for i in range(len(p))]
        p[self._drag_hit] = (p[self._drag_hit][0], p[self._drag_hit][1], 1)

        d1 = np.sqrt((p[self._drag_hit][1][0]-p[self._drag_hit-1][1][0])**2 +
                     (p[self._drag_hit][1][1]-p[self._drag_hit-1][1][1])**2)
        d2 = np.sqrt((p[self._drag_hit][1][0]-p[self._drag_hit+1][1][0])**2 +
                     (p[self._drag_hit][1][1]-p[self._drag_hit+1][1][1])**2)
        dx = p[self._drag_hit+1][1][0]-p[self._drag_hit-1][1][0]
        dy = p[self._drag_hit+1][1][1]-p[self._drag_hit-1][1][1]
        dx1 = 1/np.sqrt(dx*dx+dy*dy)*dx*(d1+d2)/2
        dy1 = 1/np.sqrt(dx*dx+dy*dy)*dy*(d1+d2)/2
        p[self._drag_hit-1] = (p[self._drag_hit-1][0],
                               (p[self._drag_hit][1][0]-dx1,
                                p[self._drag_hit][1][1]-dy1),
                               1)
        p[self._drag_hit+1] = (p[self._drag_hit+1][0],
                               (p[self._drag_hit][1][0]+dx1,
                                p[self._drag_hit][1][1]+dy1),
                               1)
        hist = self.get_root_parent().app.history
        hist.start_record()
        hist.add_history(UndoRedoFigobjMethod(self._artists[0],
                                              'pathdata', p))
        hist.stop_record()

    def onMakeAsym(self, evt):
        p = self.get_path()
        p = [p[i] for i in range(len(p))]
        p[self._drag_hit] = (p[self._drag_hit][0], p[self._drag_hit][1], 3)
        p[self._drag_hit-1] = (p[self._drag_hit-1][0],
                               p[self._drag_hit-1][1], 3)
        p[self._drag_hit+1] = (p[self._drag_hit+1][0],
                               p[self._drag_hit+1][1], 3)

        window = evt.GetEventObject().GetTopLevelParent()
        hist = GlobalHistory().get_history(window)
#        hist = self.get_root_parent().app.history
        hist.start_record()
        hist.add_history(UndoRedoFigobjMethod(self._artists[0],
                                              'pathdata', p))
        hist.stop_record()

    def isDraggable(self):
        return self._var["draggable"]

    def generate_artist(self):

        if self.isempty() is False:
            return

        if self.getp('use_var'):
            path = self.getvar("curve_path")
            trans1 = self.getvar("trans1")
            figaxes1 = self.getvar("figaxes1")
            if figaxes1 is not None:
                figaxes1 = self.find_by_full_path(str(figaxes1))

            self.set_gp_point(0, path[0][1][0], path[0][1][1],
                              trans=trans1)
            self.set_gp_figpage()
            self.set_gp_figaxes(0, figaxes1)
            self.setp('use_var', False)
            self.set_path(path)

#        if self.hasp("loaded_property"):
#            lp = self.getp("loaded_property")
#            self.add_gp(GenericPoint(0,0,trans='figure'))
#            self.set_gp_figpage()
#            self.get_gp(0).dict_to_gp(lp[-1][0], self)

        a_arr = self.make_newartist()
        # if self.hasp("loaded_property"):
        #   self.set_artist_property(a_arr[0], lp[0])
        #   self.delp("loaded_property")
        self.add_artists(a_arr)

        if not self._cb_added:
            fig_page = self.get_figpage()
            fig_page.add_resize_cb(self)
            self._cb_added = True

    def onResize(self, evt):
        self.refresh_artist()

    def do_update_artist(self):
        self.refresh_artist()

    def make_newartist(self):
        self.check_loaded_gp_data()
        width = self.getp("width")
        path = self.get_path()
        aaa = self.make_newartist0(path)
        a = aaa[0]
        lp = self.getp("loaded_property")
        if lp is not None:
            self.set_artist_property(a, lp[0])
            ec = lp[0]['edgecolor']
            for arrow in aaa[1:]:
                arrow.set_edgecolor(ec)
                arrow.set_facecolor(ec)
            self.delp("loaded_property")
        a.figobj = self
        a.figobj_hl = []
        return aaa

    def make_newartist0(self,  pin, draw_arrow=True):
        p = [(item[0], item[1]) for item in pin]
        codes, verts = zip(*p)
        path = matplotlib.path.Path(verts, codes)
        a = PathPatch(path, facecolor='none', fill=False,
                      edgecolor='black', alpha=1)
        if not draw_arrow:
            return a

        aaa = [a]
        if self.getvar('arrow1')[0]:
            dx = np.array(verts[1])-np.array(verts[0])
            dx = dx*np.sqrt(np.sum(dx**2))/1000.
            b = matplotlib.patches.FancyArrowPatch(posA=verts[0]+dx,
                                                   posB=verts[0],
                                                   connectionstyle='arc3',
                                                   arrowstyle=self.getvar(
                                                       'arrow1')[1],
                                                   mutation_scale=1,)
            aaa.append(b)
        if self.getvar('arrow2')[0]:
            dx = np.array(verts[-2])-np.array(verts[-1])
            dx = dx*np.sqrt(np.sum(dx**2))/1000.
            c = matplotlib.patches.FancyArrowPatch(posA=verts[-1],
                                                   posB=verts[-1]-dx,
                                                   connectionstyle='arc3',
                                                   arrowstyle=self.getvar(
                                                       'arrow2')[1],
                                                   mutation_scale=1,)
            aaa.append(c)
        return aaa

    def add_artists(self, a):
        figure = self.get_containter()
        self._artists = a
        figure.patches.extend(a)
        for aa in a:
            aa.set_figure(figure)

    def refresh_artist(self):
        if len(self._artists) == 0:
            return
        a1 = self._artists[0]
        z = a1.zorder
        hl = len(self._artists[0].figobj_hl) != 0
        self.del_artist(delall=True)
        a_arr = self.make_newartist()
        for a in a_arr:
            a.set_zorder(z)
        self.add_artists(a_arr)
        self.highlight_artist(hl, a_arr[0])
        a2 = self._artists[0]
        if a1 != a2:
            ifigure.events.SendReplaceEvent(self, a1=a1, a2=a2)

    def del_artist(self, artist=None, delall=False):
        #       if delall:
        artistlist = self._artists
#       else:
#           artistlist=artist

        # save_data2->load_data2 will set "loaded_property"
        self.load_data2(self.save_data2({}))
#        val = []
#        for a in self._artists:
#           val.append(self.get_artist_property(a))
#        if len(val) != 0:
#           self.setp("loaded_property", val)

        if len(artistlist) != 0:
            container = self.get_figpage()._artists[0]
            self.highlight_artist(False, artistlist)
            for a in artistlist:
                #            a is axes in this case
                #            a.remove()
                container.patches.remove(a)

        super(FigCurve, self).del_artist(artistlist)
        # print artistlist

    def highlight_artist(self, val, artist=None, path=None):

        # if artist is None:
        alist = [self._artists[0]]
        # else:
        #   alist=artist
        figure = self.get_figpage()._artists[0]
        w = figure.get_figwidth()*figure.get_dpi()
        h = figure.get_figheight()*figure.get_dpi()
#        print val, self._drag_mode, 'highlight in fig_curve'
        if val == True:
            if self._drag_mode == 3:
                return
            for a in alist:
                box = a.get_window_extent().get_points()
                x = [box[0][0], box[0][0], box[1][0], box[1][0], box[0][0]]
                y = [box[0][1], box[1][1], box[1][1], box[0][1], box[0][1]]
                hl = matplotlib.lines.Line2D(x, y, marker='s',
                                             color='k', linestyle='None',
                                             markerfacecolor='none',
                                             markeredgewidth=0.5,
                                             figure=figure)
                figure.lines.extend([hl])
                a.figobj_hl.append(hl)

                if self._drag_mode == 2:
                    if path is None:
                        path = self.get_path()
                    i = 0
                    while i < len(path):
                        item = path[i]
                        if i < len(path)-3:
                            item20 = path[i+3][0]
                        else:
                            item20 = None
                        if (item[0] == mpath.Path.MOVETO or
                                item[0] == mpath.Path.LINETO):
                            hl = [matplotlib.lines.Line2D([item[1][0]], [item[1][1]],
                                                          marker='s', markersize=5,
                                                          color='k', linestyle='None',
                                                          markerfacecolor='r',
                                                          figure=figure)]
                        elif item[0] == mpath.Path.CURVE4:
                            h0 = matplotlib.lines.Line2D(
                                [path[i-1][1][0], path[i][1][0]],
                                [path[i-1][1][1], path[i][1][1]],
                                marker=None,
                                color='k', linestyle='-',
                                markerfacecolor='k',
                                figure=figure)
                            h1 = matplotlib.lines.Line2D(
                                [path[i][1][0]],
                                [path[i][1][1]],
                                marker='o', markersize=3,
                                color='k', linestyle='None',
                                markerfacecolor='k',
                                figure=figure)
                            i = i+1
                            h2 = matplotlib.lines.Line2D(
                                [path[i][1][0], path[i+1][1][0]],
                                [path[i][1][1], path[i+1][1][1]],
                                marker=None,
                                color='k', linestyle='-',
                                markerfacecolor='k',
                                markeredgecolor='k',
                                figure=figure)
                            h3 = matplotlib.lines.Line2D(
                                [path[i][1][0]],
                                [path[i][1][1]],
                                marker='o', markersize=3,
                                color='k', linestyle='None',
                                markerfacecolor='k',
                                markeredgecolor='k',
                                figure=figure)
                            i = i+1
                            if path[i][2] == 1:
                                m = 'o'
                            elif path[i][2] == 2:
                                m = 's'
                            elif path[i][2] == 3:
                                m = '^'
                            if item20 == mpath.Path.LINETO:
                                m = 's'
                            h4 = matplotlib.lines.Line2D(
                                [path[i][1][0]],
                                [path[i][1][1]],
                                marker=m, markersize=5,
                                color='k', linestyle='None',
                                markerfacecolor='r',
                                figure=figure)
                            hl = [h0, h1, h2, h3, h4]

                        i = i+1
                        figure.lines.extend(hl)
                        a.figobj_hl.extend(hl)

#              print 'if drag_mode is xxx highlight gp here', self._drag_mode
        else:
            self._picker_a_mode = 0
            for a in alist:
                for hl in a.figobj_hl:
                    figure.lines.remove(hl)
                    a.figobj_hl = []

    def get_artist_extent(self, a):
        box = a.get_window_extent().get_points()
        return [box[0][0], box[1][0], box[0][1], box[1][1]]

    def picker_a0(self, a, evt):
        from ifigure.widgets.canvas.custom_picker import linehit_test, abs_d

        self._picker_a_mode = 0
        hit, extra, type, loc = super(FigCurve, self).picker_a0(a, evt)
        self._drag_hit = -1
        hit2, hit_seg_i = cbook.BezierHitTest(self.get_path(), evt.x, evt.y)

        if self._drag_mode == 2:
            i = 0
            for item in self.get_path():
                if ((evt.x - item[1][0])**2 +
                        (evt.y - item[1][1])**2) < 25:
                    self._drag_hit = i
                    self._drag_opath = item
                    break
                i = i+1
            if self._drag_hit != -1:
                return True, {}, 'area', 3
            self._hit_seg_i = hit_seg_i

        if self._drag_mode != 0:
            if type == 'point':
                return hit, extra, type, loc

#        x = np.transpose(a.get_verts())[0]
#        y = np.transpose(a.get_verts())[1]
#        ans, idx  = cpicker.CheckLineHit(x, y,
#                            evt.x, evt.y)
        if hit2:
            type = 'area'
            if self._drag_mode == 1:
                self._drag_mode = 2
                self._picker_a_mode = 1
                self._hit_seg_i = hit_seg_i
            else:
                if evt.button == 1:
                    self._drag_mode = 1
            return True, {}, 'area', 3
        self._drag_mode = 0
        return False, {}, type, loc

    def drag_a(self, a, evt, shift=None, scale=None):
        redraw, scale = super(FigCurve, self).drag_a(
            a, evt, shift=shift, scale=scale)
        if self._drag_hit != -1:
            self._drag_hl.set_color('None')
            self._drag_hl.set_marker('None')
            figure = self.get_containter()
            if figure.patches.count(self._drag_artist) != 0:
                figure.patches.remove(self._drag_artist)
            p = [item for item in self.get_path()]
            dx = evt.x - self._st_p[0]
            dy = evt.y - self._st_p[1]
            p = self.move_path(p, self._drag_hit, dx, dy)
            width = self.getp("width")
            self._drag_artist = self.make_newartist0(p, draw_arrow=False)

            figure.patches.append(self._drag_artist)
            self._drag_artist.set_figure(figure)
            self._drag_mode = 2
            self.highlight_artist(False)
            self.highlight_artist(True, path=p)
            self._drag_mode = 3
        return 0, scale

    def drag_a_get_hl(self, a):
        if (self._drag_hit == -1):
            return super(FigCurve, self).drag_a_get_hl(a)
        v = []
        if self._drag_artist is not None:
            v.append(self._drag_artist)
        return v+a.figobj_hl

    def dragdone_a(self, a, evt, shift=None, scale=None):
        shift = evt.guiEvent_memory.ShiftDown()
        redraw, scale0 = super(FigCurve, self).dragdone_a(a, evt,
                                                          shift=shift, scale=scale)
        if self._drag_mode == 3:
            self._drag_mode = 2

        h = []

        if self._drag_hit != -1:
            figure = self.get_containter()
            if figure.patches.count(self._drag_artist) != 0:
                figure.patches.remove(self._drag_artist)
                self._drag_artist = None
            p = [item for item in self.get_path()]
            dx = evt.x - self._st_p[0]
            dy = evt.y - self._st_p[1]
            p = self.move_path(p, self._drag_hit, dx, dy)

            self._drag_hit = -1
            h = h + [UndoRedoFigobjMethod(self._artists[0],
                                          'pathdata', p)]
            scale0 = [1, 0, 0, 1, 0, 0]
        else:
            h = h + self.scale_artist(scale0, action=True)

#        hist = self.get_root_parent().app.history
        window = evt.guiEvent_memory.GetEventObject().GetTopLevelParent()
        hist = GlobalHistory().get_history(window)
        hist.start_record()
        for item in h:
            hist.add_history(item)
        hist.stop_record()
        return 0, scale0

    def move_path(self, p, i, dx, dy):
        Path = mpath.Path
        if cbook.BezierNodeType(p, self._drag_hit) == 2:
            p[self._drag_hit] = (p[self._drag_hit][0],
                                 (p[self._drag_hit][1][0]+dx,
                                  p[self._drag_hit][1][1]+dy),
                                 p[self._drag_hit][2])
            p[self._drag_hit-1] = (p[self._drag_hit-1][0],
                                   (p[self._drag_hit-1][1][0]+dx,
                                    p[self._drag_hit-1][1][1]+dy),
                                   p[self._drag_hit-1][2])
            if self._drag_hit != len(p)-1:
                if p[self._drag_hit+1][0] == Path.CURVE4:
                    p[self._drag_hit+1] = (p[self._drag_hit+1][0],
                                           (p[self._drag_hit+1][1][0]+dx,
                                            p[self._drag_hit+1][1][1]+dy),
                                           p[self._drag_hit+1][2])
        elif cbook.BezierNodeType(p, self._drag_hit) == 3:
            p[self._drag_hit] = (p[self._drag_hit][0],
                                 (p[self._drag_hit][1][0]+dx,
                                  p[self._drag_hit][1][1]+dy),
                                 p[self._drag_hit][2])
            if self._drag_hit > 1:
                if (p[self._drag_hit-2][0] == Path.CURVE4 and
                    p[self._drag_hit-1][0] == Path.CURVE4 and
                        p[self._drag_hit-2][2] >= 1):
                    if p[self._drag_hit-1][2] == 1:
                        p[self._drag_hit-2] = (p[self._drag_hit-2][0],
                                               (p[self._drag_hit-2][1][0]-dx,
                                                p[self._drag_hit-2][1][1]-dy),
                                               p[self._drag_hit-2][2])
                    if p[self._drag_hit-1][2] == 3:
                        d = np.sqrt((p[self._drag_hit-2][1][0]-p[self._drag_hit-1][1][0])**2 +
                                    (p[self._drag_hit-2][1][1]-p[self._drag_hit-1][1][1])**2)
                        dx = p[self._drag_hit][1][0]-p[self._drag_hit-1][1][0]
                        dy = p[self._drag_hit][1][1]-p[self._drag_hit-1][1][1]
                        dx1 = 1/np.sqrt(dx*dx+dy*dy)*dx*d
                        dy1 = 1/np.sqrt(dx*dx+dy*dy)*dy*d
                        p[self._drag_hit-2] = (p[self._drag_hit-2][0],
                                               (p[self._drag_hit-1][1][0]-dx1,
                                                p[self._drag_hit-1][1][1]-dy1),
                                               p[self._drag_hit-2][2])
        elif cbook.BezierNodeType(p, self._drag_hit) == 4:
            p[self._drag_hit] = (p[self._drag_hit][0],
                                 (p[self._drag_hit][1][0]+dx,
                                  p[self._drag_hit][1][1]+dy),
                                 p[self._drag_hit][2])
            if self._drag_hit < len(p)-3:
                if (p[self._drag_hit+2][0] == Path.CURVE4 and
                    p[self._drag_hit+1][0] == Path.CURVE4 and
                        p[self._drag_hit+2][2] >= 1):
                    if p[self._drag_hit+1][2] == 1:
                        p[self._drag_hit+2] = (p[self._drag_hit+2][0],
                                               (p[self._drag_hit+2][1][0]-dx,
                                                p[self._drag_hit+2][1][1]-dy),
                                               p[self._drag_hit+2][2])
                    if p[self._drag_hit+1][2] == 3:
                        d = np.sqrt((p[self._drag_hit+2][1][0]-p[self._drag_hit+1][1][0])**2 +
                                    (p[self._drag_hit+2][1][1]-p[self._drag_hit+1][1][1])**2)
                        dx = p[self._drag_hit][1][0]-p[self._drag_hit+1][1][0]
                        dy = p[self._drag_hit][1][1]-p[self._drag_hit+1][1][1]
                        dx1 = 1/np.sqrt(dx*dx+dy*dy)*dx*d
                        dy1 = 1/np.sqrt(dx*dx+dy*dy)*dy*d
                        p[self._drag_hit+2] = (p[self._drag_hit+2][0],
                                               (p[self._drag_hit+1][1][0]-dx1,
                                                p[self._drag_hit+1][1][1]-dy1),
                                               p[self._drag_hit+2][2])
        elif cbook.BezierNodeType(p, self._drag_hit) == 1:
            p[self._drag_hit] = (p[self._drag_hit][0],
                                 (p[self._drag_hit][1][0]+dx,
                                  p[self._drag_hit][1][1]+dy),
                                 p[self._drag_hit][2])
            if (self._drag_hit == 0 and
                    self.get_closepoly()):
                p[-1] = (p[-1][0],
                         (p[0][1][0], p[0][1][1]),
                         p[-1][2])
            if (self._drag_hit == 0 and
                    p[1][0] == Path.CURVE4):
                p[1] = (p[1][0], (p[1][1][0]+dx, p[1][1][1]+dy), p[1][2])
        elif cbook.BezierNodeType(p, self._drag_hit) == 5:
            p[self._drag_hit] = (p[self._drag_hit][0],
                                 (p[self._drag_hit][1][0]+dx,
                                  p[self._drag_hit][1][1]+dy),
                                 p[self._drag_hit][2])
            if (self._drag_hit == len(p)-1 and
                    self.get_closepoly()):
                p[0] = (p[0][0],
                        (p[-1][1][0], p[-1][1][1]),
                        p[0][2])
            if (self._drag_hit == len(p)-1 and
                not self.get_closepoly() and
                    p[-2][0] == Path.CURVE4):
                p[-2] = (p[-2][0],
                         (p[-2][1][0]+dx, p[2][1][1]+dy),
                         p[-2][2])

        return p

    def scale_artist(self, scale, action=True):
        opath = self.get_path()
        minx = min([i[1][0] for i in opath])
        miny = min([i[1][1] for i in opath])
        maxx = max([i[1][0] for i in opath])
        maxy = max([i[1][1] for i in opath])
        path = []
        for item in opath:
            #           x = minx+(item[1][0]-minx)*scale[0]+(item[1][1]-miny)*scale[1]+scale[4]
            #           y = miny+(item[1][0]-minx)*scale[2]+(item[1][1]-miny)*scale[3]+scale[5]
            x = (item[1][0])*scale[0]+(item[1][1])*scale[1]+scale[4]
            y = (item[1][0])*scale[2]+(item[1][1])*scale[3]+scale[5]
            path.append((item[0], (x, y), item[2]))
        if action:
            h = []
            a1 = UndoRedoFigobjMethod(self._artists[0],
                                      'pathdata',
                                      path)
            h.append(a1)
            return h
        else:
            self.set_pathdata(path, self._artists[0])

    def set_pathdata(self, path, a):
        self.set_path(path)
        self.set_update_artist_request()

    def get_pathdata(self, a):
        return self.get_path()

    def set_closepoly(self, value, a):
        Path = mpath.Path
        if value:
            if not self.get_closepoly(a):
                p = self.get_path()
                p.append((mpath.Path.CLOSEPOLY,
                          p[0][1], 1))
                self.set_path(p)
        else:
            if self.get_closepoly(a):
                p = self.get_path()
                p = p[:-1]
                self.set_path(p)
        self.set_update_artist_request()

    def get_closepoly(self, a=None):
        Path = mpath.Path
        return bool(self.get_path()[-1][0] == mpath.Path.CLOSEPOLY)

    def get_path(self):
        gp = self.get_gp(0)
        t = gp.get_gp_transform(dir='both')
        p = self.getp("curve_path")
        return [(item[0], gp.convert_trans_p(item[1], t), item[2])
                for item in p]

    def set_path(self, path):
        gp = self.get_gp(0)
        t = gp.get_gp_transform(dir='both')
        self.setp("curve_path",
                  [(item[0], gp.convert_trans_p(item[1], None, t), item[2])
                   for item in path])

    def set_curvearrow1(self, value, a):
        self.setvar('arrow1', (value[0], value[1][0]))
        self.refresh_artist()

    def get_curvearrow1(self, a):
        return (self.getvar('arrow1')[0], (self.getvar('arrow1')[1],))

    def set_curvearrow2(self, value, a):
        self.setvar('arrow2', (value[0], value[1][0]))
        self.refresh_artist()

    def get_curvearrow2(self, a):

        return (self.getvar('arrow2')[0], (self.getvar('arrow2')[1],))



import os
import weakref
import six
if six.PY2:
    unicode = unicode
else:
    unicode = str
from collections import OrderedDict

import wx
import wx.stc as stc
#import  wx.aui as aui
use_agw = False  # for agw, onClosePage does not work....
if use_agw:
    import wx.lib.agw.aui as aui
else:
    import wx.aui as aui

import ifigure
from ifigure.utils.wx3to4 import TextEntryDialog
from ifigure.widgets.script_editor import Notebook
from ifigure.utils.edit_list import EditListPanel, EDITLIST_CHANGED
from ifigure.widgets.miniframe_with_windowlist import DialogWithWindowList
from ifigure.widgets.book_viewer import FrameWithWindowList

from ifigure.widgets.script_editor import PythonSTC
import ifigure.widgets.dialog as dialog

bitmaps = None

class DlgMdsSessionData(DialogWithWindowList):
    # class DlgMdsSessionData(FrameWithWindowList):
    def __init__(self, parent, data=None, figmds=None, cb=None, noapply=False):
        if data is None:
            return
        if figmds is None:
            return
        self._rvars = tuple()
        self.cb = cb
        self.figmds = weakref.ref(figmds, self.onLinkDead)

        style = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER
        super(DlgMdsSessionData, self).__init__(parent, wx.ID_ANY, style=style,
                                                title=self.figmds().get_full_path())
#        FrameWithWindowList.__init__(self, parent, wx.ID_ANY,
#                          title = self.figmds().get_full_path())

        if bitmaps is None:
            from ifigure.utils.cbook import make_bitmap_list
            from ifigure.ifigure_config import icondir as path
            path1 = os.path.join(path, '16x16', 'variable.png')
            path2 = os.path.join(path, '16x16', 'script.png')
            globals()['bitmaps'] = make_bitmap_list([path1, path2])


#        self.nb_big = wx.Notebook(self)
        panel1 = wx.Panel(self)

        # panel1
        elpl = [['Experiment', figmds.getvar('experiment'), 200, None],
                ['Def Node', figmds.getvar('default_node'), 200, None],
                ['Title', figmds.getvar('title'), 200, None]]

        self.elp = EditListPanel(panel1, elpl)
        self.nb = Notebook(panel1)
        self.bt_var = wx.BitmapButton(
            panel1, wx.ID_ANY, bitmaps[0])  # 'Add Variable...')
        self.bt_script = wx.BitmapButton(
            panel1, wx.ID_ANY, bitmaps[1])  # 'Add Script...')
        self.cb_local = wx.StaticText(
            panel1, wx.ID_ANY, 'Note: Script runs in main thread')
#        p = PythonSTC(self.nb, -1)
#        self.nb.AddPage(p, 'Untitiled')
        sizer = wx.BoxSizer(wx.VERTICAL)
        bsizer2 = wx.BoxSizer(wx.HORIZONTAL)
        bsizer1 = wx.BoxSizer(wx.HORIZONTAL)
        bsizer0 = wx.BoxSizer(wx.VERTICAL)

        bsizer1.Add(self.elp, 1, wx.EXPAND | wx.ALL, 3)
        bsizer1.Add(bsizer0, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 1)
        bsizer0.Add(self.bt_var, 0, wx.ALL, 0)
        bsizer0.Add(self.bt_script, 0, wx.ALL, 0)
        bsizer2.Add(self.cb_local, 1, wx.ALL, 3)

        sizer.Add(bsizer1, 0, wx.EXPAND | wx.ALL, 1)
        sizer.Add(self.nb, 1, wx.EXPAND | wx.ALL, 0)
        sizer.Add(bsizer2, 0, wx.EXPAND | wx.ALL, 1)
        panel1.SetSizer(sizer)

#        self.bt_var = wx.Button(panel1, wx.ID_ANY, 'Add Variable...')
#       self.bt_script = wx.Button(panel1, wx.ID_ANY, 'Add Script...')
#        self.cb_local = wx.CheckBox(self, wx.ID_ANY, 'Run script in main thread')

        bt_apply = wx.Button(self, wx.ID_ANY, 'Apply')
        bt_save = wx.Button(self, wx.ID_ANY, 'Save')
#        bt_reset  = wx.Button(self, wx.ID_ANY, 'Reset')
#        bt_cancel = wx.Button(self, wx.ID_ANY, 'Cancel')
#        bsizer0.Add(self.bt_var, 1, wx.ALL, 3)
#        bsizer0.Add(self.bt_script, 1, wx.ALL, 3)
#        bsizer1.Add(self.cb_local, 1, wx.ALL, 3)

        # big_panel
        self.SetSizer(wx.BoxSizer(wx.VERTICAL))
        self.GetSizer().Add(panel1, 1, wx.EXPAND)
        from ifigure.utils.wx3to4 import GridSizer
        bsizer = GridSizer(1, 5)
        bsizer.AddStretchSpacer()
        self.GetSizer().Add(bsizer, 0, wx.EXPAND | wx.ALL, 1)
        bsizer.Add(bt_save, 1, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 3)
        bsizer.AddStretchSpacer()
        bsizer.Add(bt_apply, 1, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 3)
        bsizer.AddStretchSpacer()
#        bsizer.Add(bt_reset, 1, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL, 3)
#        bsizer.Add(bt_cancel, 1, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL, 3)
        self.Bind(wx.EVT_BUTTON, self.onAddVar, self.bt_var)
        self.Bind(wx.EVT_BUTTON, self.onAddScript, self.bt_script)
        self.Bind(wx.EVT_BUTTON, self.onSave, bt_save)
#        self.Bind(wx.EVT_BUTTON, self.onCancel, bt_cancel)
        self.Bind(wx.EVT_BUTTON, self.onApply, bt_apply)
#        self.Bind(wx.EVT_BUTTON, self.onReset, bt_reset)
        if noapply:
            bt_apply.Hide()
#        self.Bind(wx.EVT_CHECKBOX, self.onHit, cb_local)
        hasscript, lc, script = self.read_script()
        if hasscript:
            self.bt_script.Enable(False)
        self.cb_local.SetLabel(self._lcstr(lc, hasscript))
        self.data2pages(data)
        self.Bind(aui.EVT_AUINOTEBOOK_PAGE_CLOSE, self.onClosePage, self.nb)
        self.Bind(aui.EVT_AUINOTEBOOK_PAGE_CHANGED,
                  self.onPageChanging, self.nb)
#        self.Bind(aui.EVT_AUINOTEBOOK_PAGE_CHANGED, self.onPageChange, self.nb)
#        wx.CallLater(1000, self.onPageChange, None)
        # self.append_help_menu()
        self.SetSize((650, 600))
        # self.SetMenuBar(self.menuBar)
        self.Layout()
        self.Show()
        self.Raise()
        # self.set_accelerator_table()
        self.nb.SetSelection(0)
        wx.GetApp().add_palette(self)
        self.Bind(wx.EVT_CLOSE, self.onClose)
        
    def onClose(self, evt):
        wx.GetApp().rm_palette(self)
        self.Destroy()
        evt.Skip()

    def onPageChanging(self, evt):
        ipage = self.nb.GetSelection()
        evt.Skip()

    def onApply(self, evt=None):
        self.onSave(self)
        fig_mds = self.figmds()
        fig_axes = fig_mds.get_figaxes()
        proj = fig_mds.get_root_parent()
        scope = proj.app.find_bookviewer(fig_mds.get_figbook())
        if scope is not None:
            scope._handle_apply_abort(allshot=True, figaxes=[fig_axes])
        if evt is not None:
            evt.Skip()

    def onOk(self, evt):
        # do something to convert texts to data
        # send action to figmds
        data, script = self.pages2data()
        self.update_figmds(data, script)
        self.data = None
        self.Destroy()

    def onCancel(self, evt):
        self.data = None
        self.Destroy()
#        if self.cb is not None:
#            self.cb()

    def onSave(self, evt=None):
        data, script = self.pages2data()
        self.update_figmds(data, script)
        ipage = self.nb.GetSelection()
        for x in range(self.nb.GetPageCount()):
            p = self.nb.GetPage(x)
            name = str(self.nb.GetPageText(x))
            if name.startswith('*'):
                name = name[1:]
            self.nb.SetPageText(x, name)
            p.SetSavePoint()

        if self.nb.GetPageCount() > ipage:
            self.nb.SetSelection(ipage)

        for x in range(self.nb.GetPageCount()):
            p = self.nb.GetPage(x)
            p.SetSavePoint()
        self.onModified(None)
#        for x in range(self.nb.GetPageCount()):
#            p=self.nb.GetPage(x)
#            self.Bind(wx.stc.EVT_STC_MODIFIED, self.onModified, p)

    def onReset(self, evt):
        fmds = self.figmds()
        if fmds is None:
            self.Destroy()
#            if self.cb is not None:
#                self.cb()
            return
#        self.Freeze()
#        self.deleteallpages()
        self.data2pages(fmds.getvar('mdsvars'))
#        self.Thaw()

    def onAddVar(self, evt):
        dlg = TextEntryDialog(self.GetTopLevelParent(),
                              "Enter the name of variable", "Add variable", "")
        if dlg.ShowModal() == wx.ID_OK:
            #            self.Freeze()
            new_name = str(dlg.GetValue())
            data, script = self.pages2data()
            if new_name in data:
                return
#            len(data.keys())
            for i in range(self.nb.GetPageCount()):
                self.nb.GetPage(i).set_syntax('none')

            p = self._new_stc(self.nb, '', syntax='none')
            self.nb.InsertPage(len(data), p, new_name, True)
            self.Bind(wx.stc.EVT_STC_MODIFIED, self.onModified, p)
#            data[new_name] = ''
#            self.data2pages(data)
        dlg.Destroy()

    def SetStatusText(self, *args, **kargs):
        pass

    def onOpenFile(self, *args, **kargs):
        pass

    def onSaveFile(self, *args, **kargs):
        # this is called from stc
        self.onApply()

    def onAddScript(self, evt=None):
        hasscript, lc, script = self.read_script()
        if hasscript:
            return

#        self.Freeze()
        data, script = self.pages2data()
        data['script.py'] = ''

        self.data2pages(data)
#        self.bt_script.Enable(False)
#        self.cb_local.SetLabel(self._lcstr(lc, True))
#        self.Thaw()
        evt.Skip()

    def _lcstr(self, value, hasscript):
        if not hasscript:
            return "note : no addtionanl python script after MDS session"
        if value:
            return "note : script runs in main thread"
        else:
            return "note : script runs in subprocess"

#    def onPageChange(self, evt=None):
#        ipage = self.nb.GetSelection()
#        label  = self.nb.GetPageText(ipage)
#        self._selected_page = label

    def onClosePage(self, evt):
        #        print("onClose")
        ipage = self.nb.GetSelection()
        label = str(self.nb.GetPageText(ipage))
        if label.startswith('*'):
            label = label[1:]
        mods = [self.nb.GetPage(x).GetModify()
                for x in range(self.nb.GetPageCount())]
        del mods[ipage]
        if str(label) in self._rvars:
            ret = dialog.message(self,
                                 '"'+label+'"' +
                                 " cannot deleted for current plot type\n(Leave it empty, if you don't need it)",
                                 'Error',
                                 0)
            evt.Veto()
            return
        npage = self.nb.GetPageCount()
        if npage == 1 or npage == 0:
            ret = dialog.message(self,
                                 '"'+label+'"' + " cannot deleted since this is the last page.",
                                 'Error',
                                 0)
            evt.Veto()
            return
        else:
            ret = dialog.message(self,
                                 'Do you want to delete "'+label+'"',
                                 'Error',
                                 2)
            if ret != 'ok':
                evt.Veto()
                return

#        if label == 'script.py' or label == '*script.py':
#            fmds = self.figmds()
#            if fmds is not None:
#              if (fmds.has_owndir() and
#                  fmds.hasvar('path')):
#                  fname = os.path.join(fmds.owndir(), fmds.getvar('path'))
#                  if os.path.exists(fname):
#                      os.remove(fname)
#                      fmds.delvar('pathmode')
#                      fmds.delvar('path')
        self.data = None
#        print label
        if str(label) == 'script.py':
            self.bt_script.Enable(True)

        wx.CallAfter(self._set_save_point, mods)

    def _set_save_point(self, mods):
        # print 'xxx', mods
        for x in range(self.nb.GetPageCount()):
            p = self.nb.GetPage(x)
            txt = self.nb.GetPageText(x)
            if not mods[x]:
                self.nb.GetPage(x).SetSavePoint()
                if txt.startswith('*'):
                    self.nb.SetPageText(x, txt[1:])
            else:
                if not txt.startswith('*'):
                    self.nb.SetPageText(x, '*' + txt)

    def update_figmds(self, data, script):
        fmds = self.figmds()
        var = self.elp.GetValue()
        fmds.setvar('experiment', str(var[0]))
        fmds.setvar('default_node', str(var[1]))
        fmds.setvar('title', str(var[2]))
        d = OrderedDict()
        for key in data:
            name = key
            if key.startswith('*'):
                name = key[1:]
            d[name] = data[key]
        fmds.applyDlgData(d)
        if script != '':
            if not fmds.has_owndir():
                fmds.mk_owndir()
            filename = 'mdsscript.py'
            fmds.setvar('pathmode', 'owndir')
            fmds.setvar('path', filename)
            fname = os.path.join(fmds.owndir(),
                                 fmds.getvar('path'))
            from ifigure.mdsplus.fig_mds import write_scriptfile
            write_scriptfile(fname, script)
        else:
            if (fmds.has_owndir() and
                    fmds.hasvar('path')):
                fname = os.path.join(fmds.owndir(), fmds.getvar('path'))
                if os.path.exists(fname):
                    os.remove(fname)
                    fmds.delvar('pathmode')
                    fmds.delvar('path')

#        fmds._script_local = self.cb_local.GetValue()
#        print self.GetParent()
        self.GetParent().property_editor.update_panel()
        # should change varviewer here too

    def checkscripttab(self):
        data = OrderedDict()
        for ipage in range(self.nb.GetPageCount()):
            name = str(self.nb.GetPageText(ipage))
            if name.startswith('*'):
                name = name[1:]
            p = self.nb.GetPage(ipage)
            data[name] = str(p.GetText())

        return 'script.py' in data

    def pages2data(self):
        data = OrderedDict()
        for ipage in range(self.nb.GetPageCount()):
            name = str(self.nb.GetPageText(ipage)).strip()
            if name.startswith('*'):
                name = name[1:]
            p = self.nb.GetPage(ipage)
            data[name] = str(p.GetText()).strip()
            p.SetSavePoint()
        script = ''
        if 'script.py' in data:
            script = data['script.py']
            del data['script.py']
        return data, script

    def data2pages(self, data):
        fmds = self.figmds()

        hasscript, lc,  script = self.read_script()
        if 'script.py' in data:
            script = data['script.py']
            hasscript = True
            del data['script.py']

        # set button
        self.cb_local.SetLabel(self._lcstr(lc, hasscript))

        # prepare pages
        npage = len([key for key in data])
        if hasscript:
            npage = npage + 1
        while self.nb.GetPageCount() != npage:
            if self.nb.GetPageCount() > npage:
                self.nb.DeletePage(self.nb.GetPageCount()-1)
            elif self.nb.GetPageCount() < npage:
                title = 'tmp_key' + str(self.nb.GetPageCount())
                p = self._new_stc(self.nb, '', syntax='python')
                title = '{:>3s}'.format(title)
                self.nb.AddPage(p, title, True)
                self.Bind(wx.stc.EVT_STC_MODIFIED, self.onModified, p)

        for ipage, key in enumerate(data):
            self.nb.SetPageText(ipage, key)
            p = self.nb.GetPage(ipage)
            if ipage == len(data)-1 and hasscript:
                pass
            else:
                p.set_syntax('none')
            self._set_stc_txt(p, data[key])

        if hasscript:
            self.nb.SetPageText(npage-1, 'script.py')
            p = self.nb.GetPage(npage-1)
            self._set_stc_txt(p, script)
            p.set_syntax('python')
            self.bt_script.Enable(False)
        else:
            self.bt_script.Enable(True)

    def read_script(self):
        from ifigure.mdsplus.fig_mds import read_scriptfile
        fmds = self.figmds()
        lc = fmds.get_script_local()
        if fmds is None:
            return
        if (fmds.has_owndir() and
                fmds.hasvar('path')):
            fname = os.path.join(fmds.owndir(), fmds.getvar('path'))
            txt = read_scriptfile(fname)
            return True,  lc, txt
        return False, lc,  ''

    def get_filelist(self):
        return self.file_list

    def onModified(self, e=None):
        ipage = self.nb.GetSelection()
#        print 'onModified', ipage
        txt = self.nb.GetPageText(ipage)
        p = self.nb.GetPage(ipage)
#        print [self.nb.GetPage(x).GetModify() for x in range(self.nb.GetPageCount())]
        # print p.GetModify()
        if not txt.startswith('*') and p.GetModify():
            self.nb.SetPageText(ipage, '*'+txt)
        if txt.startswith('*') and not p.GetModify():
            self.nb.SetPageText(ipage, txt[1:])

    def onLinkDead(self, obj):
        #        if self.cb is not None:
        #            self.cb()
        try:
            self.Destroy()
        except:
            pass

    def _new_stc(self, parent, txt, syntax='none'):
        p = PythonSTC(parent, -1, syntax=syntax)
        self._set_stc_txt(p, txt)
        p.EmptyUndoBuffer()
        p.Colourise(0, -1)
        # line numbers in the margin
        p.SetMarginType(1, stc.STC_MARGIN_NUMBER)
        #p.SetMarginWidth(1, 25)
        p.set_syntax(syntax)
        return p

    def _set_stc_txt(self, p, txt):
        #        mod = p.GetModify()
        try:
            p.SetText(txt)
#            if not mod: p.SetSavePoint()
        except UnicodeDecodeError:
            p.SetText(unicode(txt, errors='ignore'))
#            if not mod: p.SetSavePoint()
        pass



from .event_listener_proc_fake import EventListenerProc, use_event_listener
from numpy import ndarray
import multiprocessing as mp
import threading
from six.moves import queue as Queue
import traceback
import time
import tempfile
import wx
import select
from weakref import WeakKeyDictionary
import ifigure.utils.debug as debug
debug.debug_default_level = 1
dprint1, dprint2, dprint3 = debug.init_dprints('event_listener')

MDSEventReceived = wx.NewEventType()
MDSSCOPE_EVT_MDSEVENT = wx.PyEventBinder(MDSEventReceived, 1)

mds_event_listener_lock = threading.Lock()
listener_thread = None


class EventListener(threading.Thread):
    def __init__(self, *args, **kargs):
        super(EventListener, self).__init__(*args, **kargs)
        self.queue = mp.Queue()  # queue to receive message
        self.task_queue = mp.JoinableQueue()
        self.listener_proc = EventListenerProc(self.task_queue,
                                               self.queue)
        wx.CallAfter(self.listener_proc.start)
        self.viewers = WeakKeyDictionary()

    def run(self, *args, **kargs):
        while True:
            #            try:
            event_name = self.queue.get(True)
            if event_name == 'stop':
                #                from ifigure.mdsplus.mdsscope import scope_count
                #                if scope_count == 0:
                globals()['listener_thread'] = None
                return
            mds_event_listener_lock.acquire()
            check = False
            ll = list(self.viewers.items())  # protecting weakdictionary
            for v in self.viewers:
                if event_name in self.viewers[v]:
                    evt = wx.PyCommandEvent(MDSEventReceived,
                                            wx.ID_ANY)
                    evt.mdsevent_name = event_name
                    try:
                        wx.PostEvent(v, evt)
                    except:
                        check = True
                        pass
            del ll
            if check:
                del self.viewers[v]

            mds_event_listener_lock.release()


def launch_listener():
    if not use_event_listener:
        return
    if globals()['listener_thread'] is None:
        globals()['listener_thread'] = EventListener()
#        globals()['task_queue'] = mp.Queue()
        wx.CallAfter(globals()['listener_thread'].start)


def stop_listener():
    if not use_event_listener:
        return
    t = globals()['listener_thread']
    t.task_queue.put(('exit', ''), False)
    t.queue.put('stop')
    globals()['listener_thread'] = None


def disconnect_listener(viewer, event_name):
    if not use_event_listener:
        return
    t = globals()['listener_thread']
    if viewer in t.viewers:
        names = t.viewers[viewer]
        mds_event_listener_lock.acquire()
        if event_name in names:
            t.viewers[viewer].remove(event_name)

        # if noone is waiting for this event, stop event thread
        if not any([event_name in t.viewers[v]
                    for v in t.viewers]):
            t.task_queue.put(('disconnect', event_name))
        mds_event_listener_lock.release()


def connect_listener(viewer, event_name):
    if not use_event_listener:
        return
    launch_listener()
    t = globals()['listener_thread']
    t.task_queue.put(('wait', event_name))
    mds_event_listener_lock.acquire()
    if not viewer in t.viewers:
        t.viewers[viewer] = []
    if not event_name in t.viewers[viewer]:
        t.viewers[viewer].append(event_name)
    mds_event_listener_lock.release()



import wx
import weakref

from ifigure.utils.wx3to4 import menu_Append, menu_RemoveItem, menu_AppendItem
from ifigure.utils.cbook import FindFrame


class SplitterWindow(wx.SplitterWindow):
    pass


ID_LIST = {}


class PanelCheckbox(object):
    def __init__(self, panel, d):
        self.panelinfos = []
        self.child = []
        self.parent = None
        self.panel = panel
        self.visible = False
#        if self.panel.GetSizer() is None:
#            self.panel.SetSizer(wx.BoxSizer(d))
#        else:
#            self.panel.GetSizer().Add(wx.BoxSizer(d))
        self.panel.SetSizer(wx.BoxSizer(d))
        self.sash_pos = 10
        self._primary_client = None

    def onUpdateUI(self, evt):
        for pinfo, c in self.walk_tree():
            if "menu" not in pinfo:
                print(pinfo)
                continue
            if pinfo["menu"].GetId() == evt.GetId():
                if pinfo["toggle_menu"]:
                    evt.Enable(True)
                    evt.Show(True)
                else:
                    evt.Enable(False)
                    evt.Show(False)
                val = pinfo["panel"].IsShown()
                evt.Check(val)
#                print 'updating menu check', val
                return True
        return False

    def add_panel(self, cls, name, message, idx, keepsize=False, *args):
        """
           config = ("name": name,
                     "member": PanelCheckBox() or panel

        """
        info = {"name": name,
                "message": message,
                "panel": cls(self.panel),
                "panel_size": None,
                "idx": idx,
                "primary": False,
                "secondary_hidden_together": False,
                "keepsize": keepsize,
                "toggle_menu": True}
        info["panel_size"] = info["panel"].GetSize()
        self.panelinfos.append(info)
        if len(args) == 0:
            args = [1, wx.EXPAND, 0]
        self.panel.GetSizer().Add(info["panel"], *args)
        return info["panel"]

    def set_primary(self, panel):
        for pinfo in self.panelinfos:
            if pinfo["panel"] == panel:
                pinfo["primary"] = True

    def hide_toggle_menu(self, panel):
        for pinfo, c in self.walk_tree():
            #         for pinfo in self.panelinfos:
            if pinfo["panel"] == panel:
                pinfo["toggle_menu"] = False

    def show_toggle_menu(self, panel):
        for pinfo, c in self.walk_tree():
            #         for pinfo in self.panelinfos:
            if pinfo["panel"] == panel:
                pinfo["toggle_menu"] = True

    def toggle_panel(self, panel, value):
        for pinfo, c in self.walk_tree():
            if pinfo["panel"] == panel:
                if pinfo["toggle_menu"]:
                    pinfo["menu_root"].Check(pinfo["menu"].GetId(), value)
                pinfo["panel"].Show(value)
                pinfo['h'](None)
                return

    def get_toggle(self, panel):
        for pinfo, c in self.walk_tree():
            if pinfo["panel"] == panel:
                return pinfo["panel"].IsShown()
        return None

    def add_splitter(self, dsplit, dsizer):
        #        self.sp = SplitterWindow(self.panel, -1, style=wx.SP_NOBORDER|wx.SP_LIVE_UPDATE)
        self.sp = SplitterWindow(self.panel, wx.ID_ANY,
                                 style=wx.SP_NOBORDER | wx.SP_LIVE_UPDATE | wx.SP_3DSASH,
                                 #                                 style = wx.SP_LIVE_UPDATE,
                                 size=(10, 10))
#                                 style = wx.SP_LIVE_UPDATE|wx.SP_3DSASH)
#        self.sp.SetSashSize(1)
        if dsplit == 'v':
            self.split_func = self.sp.SplitVertically
        else:
            self.split_func = self.sp.SplitHorizontally

        if dsizer == 'v':
            d = wx.VERTICAL
        else:
            d = wx.HORIZONTAL

        panel1 = wx.Panel(self.sp, wx.ID_ANY, style=wx.NO_BORDER)
        panel2 = wx.Panel(self.sp, wx.ID_ANY, style=wx.NO_BORDER)
        p1 = PanelCheckbox(panel1, d)
        p2 = PanelCheckbox(panel2, d)
        p1.parent = weakref.proxy(self)
        p2.parent = weakref.proxy(self)

        self.panel.GetSizer().Add(self.sp, 1, wx.EXPAND, 0)

        self.child = (p1, p2)

        self.sp.Bind(wx.EVT_SPLITTER_SASH_POS_CHANGED, self.onSashChanged)
        self.sp.Bind(wx.EVT_SPLITTER_DOUBLECLICKED, self.onSashChanged)
        self.sp.Bind(wx.EVT_SPLITTER_UNSPLIT, self.onSashChanged)

        # minimum 50 is chosen to avoid
        #  Gtk-Critical assertion 'size >= 0' failed in GtkScrollbar
        self.sp.SetMinimumPaneSize(50)

#        self.sp.Bind(wx.EVT_SPLITTER_SASH_POS_CHANGING, self.onSashChanging)
#        self.sp.Bind(wx.EVT_SCROLLBAR, self.onSashDragged)
        return p1, p2

    def onSashChanging(self, evt):
        pass
#        print 'SashChanging'
#        if (self.child[0].panel.IsShown() and
#            self.child[1].panel.IsShown()):
#            self.sash_pos=self.sp.GetSashPosition()
#        frame = FindFrame(self.panel)
#        frame.SendSizeEvent()

    def onSashChanged(self, evt):
        if (self.child[0].panel.IsShown() and
                self.child[1].panel.IsShown()):
            self.sash_pos = self.sp.GetSashPosition()
#            print 'writing sash pos',  id(self),self.sash_pos
        if not self.child[0].panel.IsShown():
            for pinfo, c in self.child[0].walk_tree():
                pinfo["panel"].Hide()
        if not self.child[1].panel.IsShown():
            for pinfo, c in self.child[1].walk_tree():
                pinfo["panel"].Hide()

        if (not self.child[0].panel.IsShown() or
                not self.child[1].panel.IsShown()):
            self.root_parent().update_check()

        frame = FindFrame(self.panel)
        frame.SendSizeEvent()

    def root_parent(self):
        if self.parent is None:
            return self
        else:
            return self.parent.root_parent()

    def walk_tree(self):
        for c in self.child:
            for p, x in c.walk_tree():
                yield p, x
        for pi in self.panelinfos:
            yield pi, self

    def _make_menu(self, root2, pinfos):
        for i, pinfo, cb in pinfos:
            #            m = root2.Append(wx.ID_ANY, pinfo["name"],
            if not pinfo["message"] in ID_LIST:
                ID_LIST[pinfo["message"]] = wx.NewIdRef(count=1)
            id = ID_LIST[pinfo["message"]]

            m = root2.Append(id, pinfo["name"],
                             'Show '+pinfo["message"],
                             kind=wx.ITEM_CHECK)

            def menu_handler(evt, pinfo=pinfo, cb=cb):
                self.menu_handler(evt, pinfo, cb)

            pinfo["menu"] = m
            pinfo["menu_root"] = root2
            pinfo["h"] = menu_handler

    def _apply_toggle(self, root2, pinfos):
        for i, pinfo, cb in pinfos:
            id = root2.FindItem(pinfo["name"])
            if id != wx.NOT_FOUND and not pinfo["toggle_menu"]:
                menu_RemoveItem(root2, pinfo["menu"])
            elif id == wx.NOT_FOUND and pinfo["toggle_menu"]:
                menu_AppendItem(root2, pinfo["menu"])

    def append_menu(self, root, use_panel_menu=False):
        pinfos = sorted([(p[0]["idx"], p[0], p[1])
                         for p in self.walk_tree()])

        if not use_panel_menu:
            self.panel_menu = root
            self._make_menu(root, pinfos)
            self._apply_toggle(root, pinfos)
        else:
            root2 = wx.Menu()
            self.panel_menu = root2
            menu_Append(root, wx.ID_ANY, 'Panels', root2)
            self._make_menu(root2, pinfos)
            self._apply_toggle(root2, pinfos)

    def rebuild_menu(self):
        pinfos = sorted([(p[0]["idx"], p[0], p[1])
                         for p in self.walk_tree()])
        root2 = self.panel_menu
        self._apply_toggle(root2, pinfos)
#        self.update_check()

    def primary_client(self, *args):
        if len(args) == 0:
            return self.root_parent()._primary_client()
        else:
            self.root_parent()._primary_client = weakref.ref(args[0])

    def menu_handler(self, evt, pinfo=None, cb=None):
        #self == root
        app = FindFrame(pinfo["panel"])
        app.Unbind(wx.EVT_SIZE)
#        app.Freeze()
        pc = self.primary_client()
        s1 = pc.GetClientSize()
        # self.panel.Show() #### do I need this ?????

        flag = (pinfo["menu"].IsChecked() if pinfo["toggle_menu"]
                else pinfo["panel"].IsShown())
#        print pinfo["panel"]
        if flag:
            pinfo["panel"].Show()
            if pinfo["keepsize"] == 'b':
                frame = FindFrame(pinfo["panel"])
                s = frame.GetSize()
                frame.SetSize((s[0], s[1]+pinfo["panel_size"][1]))
            if pinfo["primary"]:
                for pinfo2 in cb.panelinfos:
                    if (pinfo2 is not pinfo):
                        if pinfo["secondary_hidden_together"]:
                            #                          print 'showing', pinfo2["panel"]
                            pinfo2["panel"].Show()
                        pinfo2["menu"].Enable()

        else:
            pinfo["panel"].Hide()
            pinfo["panel_size"] = pinfo["panel"].GetSize()
            if pinfo["primary"]:
                pinfo["secondary_hidden_together"] = False
                for pinfo2 in cb.panelinfos:
                    if pinfo2 is not pinfo:
                        if pinfo2["panel"].IsShown():
                            pinfo["secondary_hidden_together"] = True
                            pinfo2["panel"].Hide()
                        pinfo2["menu"].Enable(False)

        self.update_check()
        self.root_parent().set_splitters()
        s2 = pc.GetClientSize()
        p = pinfo["panel"]
        while p is not None:
            p.Layout()
            p = p.GetParent()
        app.Layout()
        s3 = pc.GetClientSize()
#        print s1, s2,  s3
        if pinfo["keepsize"] == 'b':
            frame = FindFrame(pinfo["panel"])
            s = frame.GetSize()
            frame.SetSize((s[0], s[1]-(s2[1]-s1[1])))
        elif pinfo["keepsize"] == 'r':
            frame = FindFrame(pinfo["panel"])
            s = frame.GetSize()
            frame.SetSize((s[0]-(s3[0]-s1[0]), s[1]))
        elif pinfo["keepsize"] == 'l':
            frame = FindFrame(pinfo["panel"])
            s = frame.GetSize()
            p = frame.GetPosition()
            frame.SetPosition((p[0]+(s3[0]-s1[0]), p[1]))
            frame.SetSize((s[0]-(s3[0]-s1[0]), s[1]))
        else:
            app.SendSizeEvent()
#        app.Thaw()
#        app.Raise()
        app.Bind(wx.EVT_SIZE, app.onResize)

    def bind_handler(self, obj):
        for pinfo, c in self.walk_tree():
            if not pinfo["toggle_menu"]:
                continue
            obj.Bind(wx.EVT_MENU, pinfo["h"],
                     pinfo["menu"])

    def get_sashposition(self, p=None):
        if p is None:
            p = []

        if len(self.child) != 0:
            for c in self.child:
                p = c.get_sashposition(p)
            p.append(self.sash_pos)
        return p

    def set_sashposition(self, pos, i=None):
        if i is None:
            i = 0
        if len(self.child) != 0:
            for c in self.child:
                i = c.set_sashposition(pos, i)
            self.sp.SetSashPosition(pos[i])
            if pos[i] != 0:
                self.sash_pos = pos[i]
            else:
                self.sash_pos = 10
#           print pos[i], self.sp.GetSashPosition()
            i = i+1
        return i

    def get_showhide(self):
        return [pinfo["panel"].IsShown() for pinfo, c in self.walk_tree()]

    def set_showhide(self, p, panel=None):
        i = 0
        for pinfo, c in self.walk_tree():
            if (panel is None or
                    panel == pinfo["panel"]):
                pinfo["panel"].Show(p[i])
                c.panel.Show()
                i = i + 1

    def update_check(self):
        return
        for pinfo, c in self.walk_tree():
            if not pinfo["toggle_menu"]:
                continue
            val = pinfo["panel"].IsShown()
            pinfo["menu_root"].Check(pinfo["menu"].GetId(), val)
#            pinfo["menu"].Check(val)

    def set_splitters(self):
        #        self.sash_pos = max([10, self.sash_pos])
        for c in self.child:
            c.set_splitters()
        self.visible = False
        if len(self.child) == 0:
            for pinfo in self.panelinfos:
                #              print pinfo["panel"]
                #              print pinfo["panel"].IsShown()
                if pinfo["panel"].IsShown():
                    self.visible = True
        else:
            if (self.child[0].visible and
                    self.child[1].visible):
                if not self.sp.IsSplit():
                    self.split_func(self.child[0].panel,
                                    self.child[1].panel)
                    # print self.sash_pos
                    self.sp.SetSashPosition(self.sash_pos)
#                  print 'setting sash pos', id(self), self.sash_pos
                self.visible = True
            elif self.child[0].visible:
                if self.sp.IsSplit():
                    self.sash_pos = self.sp.GetSashPosition()
#                  print 'writing sash pos2', id(self),  self.sash_pos
                    self.sp.Unsplit(self.sp.GetWindow2())
                else:
                    self.split_func(self.child[0].panel,
                                    self.child[1].panel)
                    self.sp.SetSashPosition(self.sash_pos)
                    self.sp.Unsplit(self.sp.GetWindow2())
                self.visible = True
            elif self.child[1].visible:
                if self.sp.IsSplit():
                    self.sash_pos = self.sp.GetSashPosition()
#                  print 'writing sash pos3', id(self), self.sash_pos
                    self.sp.Unsplit(self.sp.GetWindow1())
                else:
                    self.split_func(self.child[0].panel,
                                    self.child[1].panel)
                    self.sp.SetSashPosition(self.sash_pos)
                    self.sp.Unsplit(self.sp.GetWindow1())
                self.visible = True



from __future__ import print_function
#
#   fig_contour
#
#  History:
#          12.06.10  Added Highlight
#          12.06.11  Added custom picker
#          09.xx.15  Added tricontour support (object should be generated
#                    using FigTricontour)
#
#  Author :
#         Syun'ichi Shiraiwa
#  E-mail :
#         shiraiwa@psfc.mit.edu
#
#  Special note for fig_contour:
#          1) len(self._artists) can be more than 1
#
# *******************************************
#     Copyright(c) 2012- PSFC, MIT
# *******************************************
import weakref
import wx
import os
import sys
import logging
from scipy.interpolate import griddata
import numpy as np
from matplotlib.patches import Rectangle
from matplotlib.transforms import Bbox

from ifigure.utils.cbook import isiterable, isndarray, isdynamic, issequence, isnumber

import ifigure
import ifigure.events
from ifigure.mto.fig_obj import FigObj, mask_negative
from ifigure.mto.axis_user import XUser, YUser, ZUser, CUser
from ifigure.widgets.canvas.file_structure import *
import ifigure.utils.cbook as cbook
import ifigure.widgets.canvas.custom_picker as cpicker
from ifigure.utils.args_parser import ArgsParser
from ifigure.widgets.undo_redo_history import UndoRedoAddRemoveArtists, GlobalHistory
from ifigure.utils.triangulation_wrapper import tri_args

import ifigure.utils.debug as debug
dprint1, dprint2, dprint3 = debug.init_dprints('FigContour')

def_clabel_param = {
    'use_clabel': False,
    'fontsize': 10,
    'fixed_color': False,
    'colors': 'k',
    'inline': True,
    'inline_spacing': 5,
    'fmt': '%1.3f',
    'skip': 0,
    'alpha': 1}

default_kargs = {'use_tri': False,
                 'FillMode': False,
                 'interp': 'nearest',
                 'alpha':  1.0,
                 'cmap':  None, }


class FigContour(FigObj, XUser, YUser, CUser, ZUser):
    def __new__(cls, *args, **kywds):
        """
        contour : contour plot
        contour(z, n)
        contour(x, y, z, n)
        contour(z, v)
        contour(x, y, z, v)

        n: number of levels
        v: a list of contour levels
        """
        def set_hidden_vars(obj):
            if not hasattr(obj, '_tri'):
                obj._tri = None  # this can go away!?
            obj._clabels = []
            obj._clabel_param = def_clabel_param.copy()
            obj._nouse_expression = False
            obj._hit_path = None
            obj._expression = ''
            return obj

        if 'src' in kywds:
            obj = FigObj.__new__(cls, *args, **kywds)
            obj = set_hidden_vars(obj)
            return obj

        p = ArgsParser()
        p.add_opt('x', None, ['numbers|nonstr', 'dynamic'])
        p.add_opt('y', None, ['numbers|nonstr', 'dynamic'])
        p.add_var('z', ['numbers|nonstr', 'dynamic'])
        p.add_opt('n', 7, 'int')  # n = number of levels
        # v = levels (array)
        p.add_opt('v', None, ['numbers|nonstr', 'dynamic'])

        p.set_default_list(default_kargs)
        p.add_key2(("FillMode", "alpha", "use_tri"))
        p.add_key2('interp', 'str')
        p.add_key2('cmap', 'str')
        p.add_key('offset', None)
        p.add_key('zdir', 'z')

        p.set_pair("x", "y")  # x and y should be given
        # together
        p.set_ndconvert("x", "y", "z")
        p.set_squeeze_minimum_1D("x", "y", "z")

        v, kywds, d, flag = p.process(*args, **kywds)
        dprint2(v)

        if not flag:
            raise ValueError('Failed when processing argument')
            return None

        obj = FigObj.__new__(cls, *args, **kywds)
        obj = set_hidden_vars(obj)

        obj._set_expression_vars(v['v'])
        for name in ('x', 'y', 'z'):
            obj.setvar(name, v[name])
        for name in ('FillMode', 'alpha', 'n', 'v', "cmap", "offset", "zdir", "use_tri"):
            obj.setvar(name, v[name])

        if v['cmap'] is not None:
            kywds['cmap'] = v['cmap']

        obj.setvar("kywds", kywds)
        return obj

    def _set_expression_vars(self, v):
        if v is not None:
            self._expression = str(v) if not isdynamic(v) else v[1:]
            self._nouse_expression = False if self._expression != '' else True
        else:
            self._expression = ''
            self._nouse_expression = False

    def __init__(self, *args, **kywds):
        self._data_extent = None
        XUser.__init__(self)
        YUser.__init__(self)
        ZUser.__init__(self)
        CUser.__init__(self)

        self._pick_pos = None
        args = []
        if 'src' not in kywds:
            kywds = self.getvar("kywds")
        super(FigContour, self).__init__(**kywds)

    @classmethod
    def isFigContour(self):
        return True

    @classmethod
    def can_have_child(self, child=None):
        return False

    @classmethod
    def get_namebase(self):
        return 'contour'

    def property_in_file(self):
        if not self.getp('FillMode'):
            return (['linestyle', 'linewidth', 'alpha'] +
                    super(FigContour, self).property_in_file())
        else:
            return (['edgecolor', 'alpha'] +
                    super(FigContour, self).property_in_file())

    @classmethod
    def property_in_palette(self):
        return (['contour', 'labels', 'path', 'line'], [
                ["contour_fill", "contour_nlevel2", "alpha_2", "caxis"],
                ['clabel_param'],
                ['pedgecolor_2'],
                ['plinestyle_2', 'elinewidth']])

    @classmethod
    def attr_in_file(self):
        return (["v", "n", "FillMode", "alpha", "interp"] +
                super(FigContour, self).attr_in_file())

    @classmethod
    def load_classimage(self):
        from ifigure.ifigure_config import icondir as path
        idx1 = cbook.LoadImageFile(path, 'plot1.png')
        return [idx1]

    def set_parent(self, parent):
        XUser.unset_ax(self)
        YUser.unset_ay(self)
        ZUser.unset_az(self)
        CUser.unset_ac(self)
        super(FigContour, self).set_parent(parent)
        XUser.get_xaxisparam(self)
        YUser.get_yaxisparam(self)
        ZUser.get_zaxisparam(self)
        CUser.get_caxisparam(self)

    def args2var(self):
        ret = self._args2var()
        if ret:
            if 'cmap' in self.getvar("kywds"):
                cax = self.get_caxisparam()
                cax.set_cmap(self.getvar('kywds')['cmap'])
        return ret

    def _args2var(self):
        names0 = self.attr_in_file()
        names = ["x", "y", "z"] + names0
        use_np = [True]*3 + [True, True] + [False]*(len(names0)-2)
        # n and v can be dynamic too.
        values = self.put_args2var(names,
                                   use_np)
        x = values[0]
        y = values[1]
        z = values[2]
        if y is None and x is None and z.ndim == 2:
            y = np.arange((z.shape)[-2])
            self.setp("y", y)
            x = np.arange((z.shape)[-1])
            self.setp("x", x)
        if x is None:
            return False
        if y is None:
            return False
        if (x.size*y.size != z.size and
                not (x.size == z.size and y.size == z.size)):
            self.setp("x", None)
            self.setp("y", None)
            self.setp("z", None)
            return False
        return True

    def generate_artist(self):
        # this method generate artist
        # if artist does exist, update artist
        # based on the information specifically
        # managed by fig_obj tree. Any property
        # internally managed by matplotlib
        # does not change
        container = self.get_container()
        if container is None:
            return

        flag = 0
        if self.isempty() is False:
            return
        x, y, z = self._eval_xyz()  # this handles "use_var"
        if z is None:
            return

        lp = self.getp("loaded_property")

        v, n, FillMode = self.getp(('v', 'n', 'FillMode'))

        kywds = self.getvar('kywds')
        kywds['alpha'] = self.getp('alpha')

        if self.getvar('use_tri'):
            args, self._tri = tri_args(x, y, self._tri)
            args.append(np.real(z.flatten()))
        else:
            args = []
            if x is not None:
                args.append(x)
            if y is not None:
                args.append(y)
            if z is not None:
                args.append(z)
        if (not self._nouse_expression and
                v is not None):
            args.append(v)
        else:
            args.append(int(n))

        cax = self.get_caxisparam()

#           dprint2(args)
        if len(args) == 0:
            return

        self._data_extent = [np.min(x), np.max(x), np.min(y), np.max(y)]
#           self.setp("data_extent",[min(x), max(x), min(y), max(y)])
        try:
            if self.get_figaxes().get_3d():
                kywds['offset'] = self.getvar('offset')
                kywds['zdir'] = self.getvar('zdir')
                is3D = True
            else:
                is3D = False
        except:
            pass

        if self.getvar('use_tri'):
            if FillMode:
                method = container.tricontourf
            else:
                method = container.tricontour
            methodline = container.tricontour
        else:
            if FillMode:
                method = container.contourf
            else:
                method = container.contour
            methodline = container.contour

        try:
            self._mappable = method(*args, **kywds)
            self._artists = [self._mappable,]
            self.set_rasterized()
            for a in self.get_mappable():
                cax.set_crangeparam_to_artist(a)

            if self._clabel_param['use_clabel']:
               if FillMode:
                   lmappable = methodline(*args, **kywds)
               else:
                   lmappable = self._mappable
               args, kargs = self._make_clabel_param()
               self._clabels = container.clabel(lmappable, *args, **kargs)
               talpha = float(self._clabel_param['alpha'])
               for t in self._clabels:
                   t.set_alpha(talpha)
               if FillMode:
                    lmappable.remove()
                    for t in self._clabels:
                        container.add_artist(t)
               else:
                    self._clabels = []
        except Exception:
            logging.exception(
                "FigContour:generate_artist : artist generation failed")

        if is3D:
            from ifigure.matplotlib_mod.art3d_gl import quadcontourset3d_to_gl
            self._mappable = quadcontourset3d_to_gl(self._mappable)

        if lp is not None:
            #              print lp
            for i, var in enumerate(lp):
                if len(self._artists) > i:
                    self.set_artist_property(self._artists[i], var)
            self.delp("loaded_property")

        for path in self._artists:
            #                 path.set_picker(cpicker.Picker)
            path.figobj = self
            path.figobj_hl = []
            path.set_zorder(self.getp('zorder'))


    def del_artist(self, artist=None, delall=False):
        #
        #  delete highlight and artist
        #  for fig_contour, all artists are treated
        #  as one set. it does not support to delete
        #  a part of path collections
        delall = True
        if delall:
            artistlist = self._artists
        else:
            artistlist = self._artists

        self.store_loaded_property()

        for t in self._clabels:
            t.remove()
        self._clabels = []

        for a in artistlist:
            a.remove()

        self._mappable = None

        super(FigContour, self).del_artist(artistlist)

    def get_mappable(self):
        return [self._mappable]

    def reset_artist(self):
        #print('resetting contour artist')
        self.del_artist(delall=True)

        self.setp('use_var', True)
        self.generate_artist()

    def highlight_artist(self, val, artist=None):
        from ifigure.matplotlib_mod.art3d_gl import Poly3DCollectionGL
        from ifigure.matplotlib_mod.art3d_gl import Line3DCollectionGL
        figure = self.get_figpage()._artists[0]
        ax = self.get_figaxes()
        if artist is None:
            alist = self._artists
        else:
            alist = artist

        if val == True:
            if self._parent is None:
                return
            container = self.get_container()
            if container is None:
                return

            if isinstance(alist[0], Poly3DCollectionGL):
                hl = alist[0].add_hl_mask()
                for item in hl:
                    alist[0].figobj_hl.append(item)
            else:
                de = self.get_data_extent()
                x = (de[0], de[1], de[1], de[0], de[0])
                y = (de[2], de[2], de[3], de[3], de[2])

                facecolor = 'k'

                if hasattr(self, "_hit_path3d") and self._hit_path3d is not None:
                    v = self._hit_path3d[0]
                    hl = container.plot(v[:,0], v[:,1], v[:,2], marker='s',
                                        color='k', linestyle='None',
                                        markerfacecolor='None',
                                        markeredgewidth=0.5,
                                        scalex=False, scaley=False)
                    alist[0].add_hl_mask()
                else:
                    v = self._hit_path.vertices
                    hl = container.plot(v[:, 0], v[:, 1], marker='s',
                                        color='k', linestyle='None',
                                        markerfacecolor='None',
                                        markeredgewidth=0.5,
                                        scalex=False, scaley=False)

                for item in hl:
                    alist[0].figobj_hl.append(item)

                '''
                hlp = Rectangle((de[0], de[2]),
                                de[1]-de[0],
                                de[3]-de[2],
                                alpha=0.3, facecolor=facecolor,
                                figure=figure,
                                transform=container.transData)
                if ax is not None:
                    x0, y0 = ax._artists[0].transAxes.transform((0, 0))
                    x1, y1 = ax._artists[0].transAxes.transform((1, 1))
                    bbox = Bbox([[x0, y0], [x1, y1]])
                    hlp.set_clip_box(bbox)
                    hlp.set_clip_on(True)
                figure.patches.append(hlp)
                alist[0].figobj_hl.append(hlp)
                '''
        else:
            for a in alist:
                if len(a.figobj_hl) == 0:
                    continue
                for hl in a.figobj_hl:
                    hl.remove()
                a.figobj_hl = []
#
#   Setter/Getter
#
    def set_contour_nlevel2(self, value, a=None):
        self._nouse_expression = not value[0]
#        print value
        self.setp('n', int(value[2][0]))
        self.setp('v', value[1][0][1])
        self._expression = value[1][0][0]
        self.highlight_artist(False)
        self.del_artist(delall=True)
        self.delp('loaded_property')
        self.generate_artist()

        sel = [weakref.ref(self._artists[0])]
        import wx
        app = wx.GetApp().TopWindow
        ifigure.events.SendSelectionEvent(self, w=app, selections=sel)

    def get_contour_nlevel2(self, a):
        v = (not self._nouse_expression,
             [(self._expression, self.getp('v'))],
             [str(self.getp('n'))], )
#        print 'returning figobj value', v
        return v

    def set_contour_fillmode(self, value, a):
        self.setp('FillMode', value)
        self.del_artist(delall=True)
        self.delp('loaded_property')
        self.generate_artist()

        sel = [weakref.ref(self._artists[0])]
        import wx
        app = wx.GetApp().TopWindow
        ifigure.events.SendSelectionEvent(self, w=app, selections=sel)

    def get_contour_fillmode(self, a):
        return self.getp('FillMode')

    def set_alpha(self, value, a):
        for a in self._artists:
            a.set_alpha(value)

    def get_alpha(self, a):
        return self._artists[0].get_alpha()

    def set_edgecolor(self, value, a):
        for a in self._artists:
            a.set_edgecolor(value)

    def get_edgecolor(self, a):
        v = a.get_edgecolor()
        return v

    def set_elinewidth(self, value, a):
        for a in self._artists:
            a.set_linewidth(value)

    def get_elinewidth(self, a):
        v = a.get_linewidth()
        if len(v) > 0:
            return v[0]
        return ['1.0']

    def set_linestyle(self, value, a):
        for a in self._artists:
            a.set_linestyle(value)

    def get_linestyle(self, a):
        v = a.get_linestyle()
        return v

#
#   HitTest
#
    def picker_a(self, artist, evt):
        from matplotlib.collections import PathCollection
        from matplotlib.collections import LineCollection
        from ifigure.matplotlib_mod.art3d_gl import Poly3DCollectionGL
        from ifigure.widgets.canvas.custom_picker import CheckLineHit
        axes = artist.axes
        if axes is None:
            return False, {}
        trans = axes.transData
        self._hit_path = None
        if isinstance(artist, Poly3DCollectionGL):
            hit, extra = artist.contains(evt)
            if hit:
                return True, {'child_artist': artist}
            else:
                return False, {}
        else:
            for k, path in enumerate(artist.get_paths()):
                xy = trans.transform(path.vertices)
                ans, hit = CheckLineHit(xy[:, 0], xy[:, 1], evt.x, evt.y)

                if ans:
                    self._hit_path = path
                    self._hit_path3d = None
                    if hasattr(artist, "_3dverts_codes"):
                        self._hit_path3d = artist._3dverts_codes[k]
                    return True, {'child_artist': artist}
#                if path.contains_point((evt.x, evt.y), transform=trans, radius=6):
#                     self._hit_path = path
#                     return True, {'child_artist':artist}
            return False, {}
        # following is old version
        # should not come here..
        printd('contour has artist nither PathCollection nor LienCollection... Why??')
        if axes is None:
            return False, {}
        de = self.get_data_extent()
        if (evt.xdata > de[0] and
            evt.xdata < de[1] and
            evt.ydata > de[2] and
                evt.ydata < de[3]):
            return True,  {'child_artist': artist}
        return False, {}

    def picker_a0(self, artist, evt):
        hit, extra = self.picker_a(artist, evt)
        if hit:
            return hit, extra, 'area', 3
        else:
            return False, {}, None, 0

#
#  Popup in Canvas
#
#    def canvas_menu(self):
#        return [("Export",  self.onExport, None)] + \
#                super(FigContour,self).canvas_menu()
    def canvas_menu(self):
        ac = self.get_caxisparam()
        if ac is None:
            return super(FigContour, self).tree_viewer_menu()
        if ac.cbar_shown():
            m = [('Hide color bar', self.onHideCB1, None), ]
        else:
            m = [('Show color bar', self.onShowCB1, None), ]
        menus = [("Show slice",  self.onSlice, None),
                 ("Generate path plot",   self.onCopyPath, None)] + m + \
            [("Export path",   self.onExportPath, None)] + \
            super(FigContour, self).canvas_menu()
        return menus

    def onExportPath(self, event):
        if self._hit_path is not None:
            if hasattr(self, "_hit_path3d") and self._hit_path3d is not None:
                v = self._hit_path3d[0]
                fig_val = {"xdata": v[:, 0],
                          "ydata": v[:, 1],
                          "zdata": v[:, 2]}
                text = '#Exporting data as fig_val[\'xdata\'], fig_val[\'ydata\'], fig_val[\'zdata\']'
            else:
                v = self._hit_path.vertices
                fig_val = {"xdata": v[:, 0],
                           "ydata": v[:, 1]}
                text = '#Exporting data as fig_val[\'xdata\'], fig_val[\'ydata\']'
            self._export_shell(fig_val, 'fig_val', text)

    def onCopyPath(self, event):
        from ifigure.mto.fig_plot import FigPlot
        from ifigure.mto.fig_fill import FigFill
        if self._hit_path is not None:
            canvas = event.GetEventObject()
            objs = []

            if hasattr(self, "_hit_path3d") and self._hit_path3d is not None:
               v = self._hit_path3d[0]
               code = self._hit_path3d[1]
               idx = np.where(code == 1)[0]

               vx = v[:,0]
               vy = v[:,1]
               vz = v[:,2]
               for xx, yy, zz in zip(np.split(vx, idx),
                                     np.split(vy, idx),
                                     np.split(vz, idx)):
                   if len(xx) == 0:
                       continue
                   objs.append(FigPlot(xx, yy, zz, 'k'))

            else:
               v = self._hit_path.vertices
               code = self._hit_path.codes

               idx = np.where(code == 1)[0]

               vx = v[:,0]
               vy = v[:,1]

               for xx, yy in zip(np.split(vx, idx),
                                 np.split(vy, idx)):
                   if len(xx) == 0:
                       continue
                   objs.append(FigPlot(xx, yy, 'k'))

            ax = self.get_figaxes()
            name = ax.get_next_name('Contour_path')
            for obj in objs:
                ax.add_child(name, obj)
                obj.realize()

            ax.set_bmp_update(False)
            canvas.draw()

            artists = [weakref.ref(obj._artists[0])]
            h = [UndoRedoAddRemoveArtists(artists=artists,
                                          mode=0)]
            window = canvas.GetTopLevelParent()
            GlobalHistory().get_history(window).make_entry(h, menu_name='add path plot')

    def onExport(self, event):
        from matplotlib.artist import getp

        shell = self.get_root_parent().app.shell
        canvas = event.GetEventObject()
        sel = [a() for a in canvas.selection]
        for a in self._artists:
            if a in sel:
                print("Exporting Data to Shell")
                x, y, z = self._eval_xyz()
                fig_val = {"zdata": z,
                           "xdata": x,
                           "ydata": y}
                text = '#Exporting data as fig_val[\'xdata\'], fig_val[\'ydata\'], fig_val[\'zdata\']'
                self._export_shell(fig_val, 'fig_val', text)
                break
        # print "exporting data to file"

    def get_slice(self, xin, yin):
        x, y, z = self._eval_xyz()
        if z.ndim != 2:
            return None, None
        X, Y = np.meshgrid(x, y)
        p1 = np.transpose(np.vstack((X.flatten(), Y.flatten())))

        x0 = xin+np.zeros(y.size)
        y0 = yin+np.zeros(x.size)

        interp = self.getp("interp")
        zp1 = griddata(p1, z.flatten(),
                       (x, y0),
                       method=interp)
        zp2 = griddata(p1, z.flatten(),
                       (x0, y),
                       method=interp)
        return (x, zp1), (y, zp2)

    def onSlice(self, event):
        from ifigure.interactive import figure, plot, nsec, isec, update, title, xlabel, ylabel
        if event.mpl_xydata[0] is None:
            return

        app = self.get_root_parent().app
        data1, data2 = self.get_slice(event.mpl_xydata[0],
                                      event.mpl_xydata[1])
        if data1 is None:
            return
        if data2 is None:
            return
        from ifigure.widgets.book_viewer import BookViewer
        book, viewer = app.open_newbook_in_newviewer(BookViewer)
        book.get_page(0).set_section(2)
        ou = update()
        isec(0)
        plot(data1[0], data1[1])
        title('x slice : y = '+str(event.mpl_xydata[1]))
        xlabel('x')
        isec(1)
        plot(data2[0], data2[1])
        title('y slice : x = '+str(event.mpl_xydata[0]))
        xlabel('y')
        update(ou)

#
#   extent
#
    def get_data_extent(self):
        if self._data_extent is not None:
            return self._data_extent
        x = self.eval("x", np=True)
        y = self.eval("y", np=True)
        if (x is None or
                y is None):
            z = self.eval("z", np=True)
            if (z is not None):
                x = np.arange(z.shape[1])
                y = np.arange(z.shape[0])
                self._data_extent = [
                    np.min(x), np.max(x), np.min(y), np.max(y)]
        else:
            self._data_extent = [np.min(x), np.max(x), np.min(y), np.max(y)]
        return self._data_extent

#
#   range
#
    def get_xrange(self, xrange=[None, None], scale='linear'):
        if (self.getvar('offset') is not None and
                self.getvar('zdir') == 'x'):
            return self._update_range(xrange, (self.getvar('offset'),
                                               self.getvar('offset'),))
        x, y, z = self._eval_xyz()
        if x is None:
            return
        if scale == 'log':
            x = mask_negative(x)
        return self._update_range(xrange, [np.nanmin(x), np.nanmax(x)])

    def get_yrange(self, yrange=[None, None],
                   xrange=[None, None], scale='linear'):
        #        de = self.get_data_extent()
        if (self.getvar('offset') is not None and
                self.getvar('zdir') == 'y'):
            return self._update_range(yrange, (self.getvar('offset'),
                                               self.getvar('offset'),))
        x, y, z = self._eval_xyz()
        if x is None:
            return
        if y is None:
            return
        if scale == 'log':
            y = mask_negative(y)
        return self._update_range(yrange, (np.nanmin(y), np.nanmax(y)))

    def get_crange(self, crange=[None, None],
                   xrange=[None, None],
                   yrange=[None, None], scale='linear'):

        x, y, z = self._eval_xyz()
        if (x is None or
                y is None):
            x = np.arange(z.shape[1])
            y = np.arange(z.shape[0])
        if (self.getvar('offset') is not None and
            (self.getvar('zdir') == 'x' or
             self.getvar('zdir') == 'y')):
            crange = self._update_range(crange, (np.amin(z), np.amax(z)))
        elif (xrange[0] is not None and
              xrange[1] is not None and
              yrange[0] is not None and
              yrange[1] is not None):
            zt = np.ma.masked_array(z)
            if zt.ndim == 2:
                if y.ndim == 1:
                    zt[(y < yrange[0]) | (y > yrange[1]), :] = np.ma.masked
                else:
                    zt[(y < yrange[0]) | (y > yrange[1])] = np.ma.masked
                if x.ndim == 1:
                    zt[:, (x < xrange[0]) | (x > xrange[1])] = np.ma.masked
                else:
                    zt[(x < xrange[0]) | (x > xrange[1])] = np.ma.masked
            else:
                zt[(y < yrange[0]) | (y > yrange[1])] = np.ma.masked
                zt[(x < xrange[0]) | (x > xrange[1])] = np.ma.masked

            if scale == 'log':
                zt[z <= 0] = np.ma.masked
            crange = self._update_range(crange, (np.amin(zt), np.amax(zt)))

        return crange

    def get_zrange(self, zrange=[None, None],
                   xrange=[None, None],
                   yrange=[None, None], scale='linear'):
        if (self.getvar('offset') is not None and
                self.getvar('zdir') == 'z'):
            return self._update_range(zrange, (self.getvar('offset'),
                                               self.getvar('offset'),))
        else:
            zrange = self.get_crange(crange=zrange,
                                     xrange=xrange,
                                     yrange=yrange, scale='linear')

        return zrange

    def save_data2(self, data=None):
        def check(obj, name):
            if not isinstance(obj.getp(name), np.ndarray):
                return False
            if not isinstance(obj.getvar(name), np.ndarray):
                return False
            return obj.getp(name) is obj.getvar(name)

        if data is None:
            data = {}
        var = {'x': check(self, 'x'),
               'y': check(self, 'y'),
               'z': check(self, 'z')}

        if not var["x"]:
            if self._save_mode == 0:
                var["xdata"] = self.getp("x")
            else:
                var["xdata"] = np.array([0, 1])
        if not var["y"]:
            if self._save_mode == 0:
                var["ydata"] = self.getp("y")
            else:
                var["ydata"] = np.array([0, 1])
        if not var["z"]:
            if self._save_mode == 0:
                var["zdata"] = self.getp("z")
            else:
                var["zdata"] = np.arange(4).reshape(2, 2)

        var["n"] = self.getp("n")
        var["v"] = self.getp("v")
        var["nouse_expression"] = self._nouse_expression
        var["clabel_param"] = self._clabel_param
        dprint2('save_data2', var)
        data['FigContour'] = (1, var)
        data = super(FigContour, self).save_data2(data)
        return data

    def load_data2(self, data):
        d = data['FigContour']
        super(FigContour, self).load_data2(data)
        dprint2('load_data2', d[1])
        var = d[1]
        names = ["x", "y", "z"]
        for name in names:
            if var[name]:
                self.setp(name,
                          self.getvar(name))
            else:
                self.setp(name, var[name+'data'])
        self.setp('n', var["n"])
        self.setp('v', var["v"])
        if "nouse_expression" in var:
            self._nouse_expression = var["nouse_expression"]
        if "clabel_param" in var:
            self._clabel_param = var["clabel_param"]

    def prepare_compact_savemode(self):
        var_bk = self._var.copy()
        self._var['z'] = np.arange(4).reshape(2, 2)
        self._var['x'] = np.array([0, 1])
        self._var['y'] = np.array([0, 1])
        return var_bk

    def _eval_xyz(self):
        if self.getp('use_var'):
            success = self.handle_use_var()
            if not success:
                return None, None, None
            self._set_expression_vars(self.getvar('v'))
        return self.getp(("x", "y", "z"))
#
#   clabel
#

    def _clabal_gui2kwargs(self, gui_param):
        self._clabel_param['use_clabel'] = gui_param[0]
        self._clabel_param['fontsize'] = gui_param[1][0]
        self._clabel_param['fixed_color'] = gui_param[1][1][0]
        self._clabel_param['colors'] = gui_param[1][1][1]
        self._clabel_param['inline'] = gui_param[1][2]
        self._clabel_param['inline_spacing'] = float(gui_param[1][3])
        self._clabel_param['fmt'] = str(gui_param[1][4])
        self._clabel_param['skip'] = int(gui_param[1][5])
        self._clabel_param['alpha'] = float(gui_param[1][6])

    def _make_clabel_param(self):
        keys = ['fontsize', 'colors', 'inline', 'inline_spacing', 'fmt']
        kargs = {key: self._clabel_param[key] for key in keys}
        if not self._clabel_param['fixed_color']:
            kargs['colors'] = None
        if self._clabel_param['skip'] == 0:
            args = tuple()
        else:
            k = int(self._clabel_param['skip'])
            args = (self._mappable.levels[:(
                self._mappable.levels.size//k)*k].reshape(-1, k)[:, 0],)
        kargs['use_clabeltext'] = True
        return args, kargs

    def set_clabel_param(self, value, a):
        self._clabal_gui2kwargs(value)
        self.del_artist(delall=True)
        self.delp('loaded_property')
        self.generate_artist()
        sel = [weakref.ref(self._artists[0])]
        import wx
        app = wx.GetApp().TopWindow
        ifigure.events.SendSelectionEvent(self, w=app, selections=sel)

    def get_clabel_param(self, a):
        v = (self._clabel_param['use_clabel'],
             [self._clabel_param['fontsize'],
                 (self._clabel_param['fixed_color'],
                  self._clabel_param['colors']),
                 self._clabel_param['inline'],
                 float(self._clabel_param['inline_spacing']),
                 str(self._clabel_param['fmt']),
                 str(self._clabel_param['skip']),
                 str(self._clabel_param['alpha']), ])

        return v
        # should return like  (False, [8.0, (False, [(1.0, 0.0, 0.0, 1.0)]),
        # True, 5.0, u'%1.3f', '0'])



from __future__ import print_function
import os
import wx
import logging
import shutil
import weakref
import ifigure
import ifigure.events
import ifigure.utils.cbook as cbook
import ifigure.mto.abs_module as abs_module
import ifigure.mto.py_code as py_code
import ifigure.add_on
import ifigure.widgets.dialog as dialog
from ifigure.mto.fileholder import FileHolder
import ifigure.utils.debug as debug
dprint1, dprint2, dprint3 = debug.init_dprints('PyModule')

#
#  the same as PyCode but different default icon
#
active_module = {}


class PyModule(py_code.PyCode, FileHolder):
    def __init__(self, *args, **kargs):
        self._obj = None
        self._can_have_child = False
        self._has_private_owndir = False
        self._first_load = True
        self._args = args
        self._kargs = kargs
        super(PyModule, self).__init__(*args, **kargs)

    @classmethod
    def isPyModule(self):
        return True

    @classmethod
    def get_namebase(self):
        return 'module'

    @classmethod
    def load_classimage(self):
        from ifigure.ifigure_config import icondir as path
        idx1 = cbook.LoadImageFile(path, 'cog.png')
        return [idx1]

    def pv_kshortcut(self, key):
        '''
        define keyborad short cut on project viewer
        must return a method responding key
        '''
        if key == wx.WXK_RETURN:
            return self.onEditModule
        return None

    def __getitem__(self, key):
        if hasattr(self._obj, '_getitem_'):
            return self._obj._getitem_(key)
        else:
            return super(py_code.PyCode, self).__getitem__(key)

    def destroy(self, clean_owndir=True):
        if self._obj is not None:
            if hasattr(self._obj, 'clean'):
                self._obj.clean()
            self._obj.unbind_method()
        self._obj = None
        super(py_code.PyCode, self).destroy(clean_owndir=clean_owndir)

    def can_have_child(self, child=None):
        return self._can_have_child

    def load_customimage(self):
        from ifigure.ifigure_config import icondir as path
        idx1 = cbook.LoadImageFile(path, self._obj._icon)
#       print 'loading custom icon'
        return [idx1]

    def do_run(self, e=None):
        if self._suppress is False:
            return self._module.run_func('onRun', self)

    def call(self, name, *args, **kargs):
        '''
        call module function/method :
             call(fname, *args, **kargs)
        '''
        if hasattr(self._obj, name):
            return self.call_method(name,  *args, **kargs)
        else:
            return self.call_function(name, *args, **kargs)

    def has_method(self, mname):
        return self._obj.has_method(mname)

    def call_method(self, mname, *args, **kargs):
        # equivalent to call _obj.mname(...)
        # but can call method by string
        logging.basicConfig(level=logging.DEBUG)
        try:
            return self._obj.run_method(mname, *args, **kargs)
        except Exception:
            raise ValueError
#            logging.exception("Module Method Call Failed")

    def call_function(self, fname, *args, **kargs):
        logging.basicConfig(level=logging.DEBUG)
        try:
            f = getattr(self._obj._m_co, fname)
            return f(*args, **kargs)
        except Exception:
            raise ValueError
#            logging.exception("Module Method Call Failed")

    def load_module(self, file):
        try:
            self.set_path_pathmode(file)
            self._obj = abs_module.AbsModule(file=file, obj=self)
            if self._obj._icon != '':
                self._use_custom_image = True
                self._custom_image_load_done = False
            else:
                self._use_custom_icon = False

            # self._name=self._obj._m_co.class_name
            if self._parent is not None:
                names = self._parent.get_childnames()
                if names.count(self._obj._m_co.class_name) != 0:
                    self._name = self.get_parent().get_next_name(self._obj._m_co.class_name)

            self._can_have_child = True
            self._has_private_owndir = True
            allow_only_one_active = False
            if hasattr(self._obj._m_co, 'can_have_child'):
                self._can_have_child = self._obj._m_co.can_have_child
            if hasattr(self._obj._m_co, 'has_private_owndir'):
                self._has_private_owndir = self._obj._m_co.has_private_owndir
            if hasattr(self._obj._m_co, 'nosave_var0'):
                self._nosave_var0 = self._obj._m_co.nosave_var0

            if hasattr(self._obj._m_co, 'allow_only_one_active'):
                allow_only_one_active = self._obj._m_co.allow_only_one_active
            if allow_only_one_active:
                if not file in active_module:
                    active_module[file] = []
                if not self in [m() for m in active_module[file] if m() is not None]:
                    if any([not m().is_suppress() for m in active_module[file] if m() is not None]):
                        self._status = 'off'
                        self.set_suppress(True)
                    active_module[file].append(weakref.ref(self))

            if self._first_load:
                if hasattr(self._obj, 'init'):
                    self._obj.init(*self._args, **self._kargs)
                self._first_load = False
        except:
            logging.exception("Module Method Call Failed")
            print(("PyModule: load_module failed:", file))
        return self

    def activate(self):
        file = self.path2fullpath()
        if file in active_module:
                # cleaning list first
            active_module[file] = [
                m for m in active_module[file] if m() is not None]
            for m in active_module[file]:
                m().set_suppress(True)
                m()._status = 'off'
            self.set_suppress(False)
            self._status = 'on'

    def tree_viewer_menu(self):
     # return MenuString, Handler, MenuImage
        menu = []
        if self._obj is None:
            try:
                self.init_after_load()
            except Exception:
                logging.exception("Module Loading Failed")

        try:
            if self._obj is not None:
                if self._obj.check_filenew():
                    self._obj.load_module()
                #
                if self.has_method('tree_viewer_menu'):
                    list = self.call_method('tree_viewer_menu')
                else:
                    list = self._obj.menu
                for item in list:
                    txt = item[0]
                    mname = item[1]

                    def xxx(e, method=mname, do_skip=item[2]):
                        self._obj.run_method(method, e)
                        if do_skip:
                            e.Skip()
                    menu.append((txt, xxx, None))
        except Exception:
            logging.exception("Module Loading Failed")

        if self._obj is not None:
            d = self._obj.get_debug()
        else:
            d = 0
        a = ['0', '1', '2']
        a[d] = '-'+a[d]
        return menu + \
            [('---', None, None),
             ('+Module', None, None),
                ('Edit File', self.onEditModule, None),
                ('Select File', self.onSelectModuleFile, None),
                ('Import File', self.onImportModuleFile, None),
                ('Export File', self.onExportModuleFile, None),
                ('+Set Degug Level', None, None),
                (a[0], self.onDebug0, None),
                (a[1], self.onDebug1, None),
                (a[2], self.onDebug2, None),
                #               ('Import File', self.onImportScriptFile, None),
                ('!', None, None),
                ('!', None, None),
                ('---', None, None)] + \
            super(PyModule, self).tree_viewer_menu()

    def onDebug0(self, e):
        self._obj.set_debug(0)

    def onDebug1(self, e):
        self._obj.set_debug(1)

    def onDebug2(self, e):
        self._obj.set_debug(2)

#    def onProjTreeActivate(self, e):
#        self.onEditModule(e)
    def onEditModule(self, e):
        print(wx.GetKeyState(wx.WXK_CONTROL))
#        if wx.GetKeyState(wx.WXK_CONTROL):
#           self._obj.edit_module()
#           return
#        handler=self.get_root_parent().app.GetEventHandler()
#        evt=ifigure.events.TreeDictEvent(
#                    ifigure.events.EditFile,
#                    wx.ID_ANY)
#        evt.SetTreeDict(self)
        if self._obj is None:  # if compilation failed _obj is none
            fpath = self.path2fullpath()
        else:
            fpath = self._obj._m_file
        ifigure.events.SendEditFileEvent(self,
                                         w=e.GetEventObject(),
                                         file=fpath)
#        handler.ProcessEvent(evt)

    def onSelectModuleFile(self, e=None):
        open_dlg = wx.FileDialog(None, message="Select module",
                                 wildcard='*.py', style=wx.FD_OPEN)
        if open_dlg.ShowModal() != wx.ID_OK:
            open_dlg.Destroy()
            raise ValueError
        file = open_dlg.GetPath()
        self.load_module(file)
        open_dlg.Destroy()

    def onImportModuleFile(self, e=None):
        file = dialog.read(None, message="Select Module",
                           wildcard='*.py')
        mode, path = self.fullpath2path(file)
        if (mode == 'wdir' or mode == 'owndir'):
            m = 'Import should import from somewhere outside project directory',
            ret = dialog.message(None, message=m,
                                 title='Import error')
            return
        self.import_module(file)

    def onExportModuleFile(self, e=None):
        opath = self.path2fullpath()
        file = dialog.write(None, defaultfile=opath,
                            message="Select script",
                            wildcard='*.py')
        if file == '':
            return
        mode, path = self.fullpath2path(file)
        if (mode == 'wdir' or mode == 'owndir'):
            m = 'Export should export to somewhere outside project directory',
            ret = dialog.message(None, message=m,
                                 title='Import error')
            return

        shutil.copyfile(opath, file)

    def import_module(self, file):
        '''
        import *.py file from outside of wdir
        the location of original file will be
        saved to "orgpath", "orgpathmode"
        '''
        mode, path = self.fullpath2path(file)
        if mode == 'wdir':
            # do nothing in this case
            print('can not import from wdir')
            return
        if mode == 'owndir':
            # do not write inside proj directory
            print('can not import from owndir')
            return

        self.setvar("orgpath", str(path))
        self.setvar("orgpathmode", str(mode))

        if not self.has_owndir():
            self.mk_owndir()
        file2 = os.path.join(self.owndir(), self.name+'.py')
        shutil.copyfile(file, file2)

        self.load_module(file2)

    def init_after_load(self, olist=None, nlist=None):
        #        print "loading", self.getvar("module file")
        name = self._name
        file = self.path2fullpath()
        if file != '':
            self.load_module(file)
        self._name = name

        if hasattr(self._obj, 'init_after_load'):
            return self.call_method('init_after_load', olist, nlist)

    def rename(self, new,  ignore_name_readonly=False):
        osfile = self.path2fullpath()
        oname = self.name
        oowndir = self.owndir()
        super(PyModule, self).rename(
            new, ignore_name_readonly=ignore_name_readonly)

        sfile = self.path2fullpath()
        mode = self.getvar("pathmode")
        if mode == 'owndir':
            nsfile = os.path.join(self.owndir(), self.name+'.py')
            if os.path.exists(sfile):
                os.rename(sfile, nsfile)
                self.set_path_pathmode(nsfile)
            else:
                print(('can not find', sfile))

            # remove old pyc pyo
            if os.path.exists(sfile+'c'):
                os.remove(sfile+'c')
            if os.path.exists(sfile+'o'):
                os.remove(sfile+'o')
        else:
            nsfile = sfile
        param = {"oldname": osfile, "newname": nsfile}
        op = 'rename'
        file = self.path2fullpath()
        if file != '':
            self.load_module(file)
        ifigure.events.SendFileChangedEvent(self, operation=op,
                                            param=param)

        return
        # 2013 11 ( followin is an old routine)
        if (mode == 'wdir'):
            # 2012. 08. 24  this should not happen.
            # py_module should always
            # use pathmode=owndir or
            # outside of wdir
            print("pathmode is not owndir")
            print("rename may not work properly")
            return
        if (oname != self.name and
                mode == 'owndir'):
            osfile = os.path.join(oowndir, self.getvar("path"))
            sfile = os.path.join(self.owndir(), self.getvar("path"))
            nsfile = os.path.join(self.owndir(), self.name+'.py')

            if os.path.exists(sfile):
                os.rename(sfile, nsfile)
                self.set_path_pathmode(nsfile)
                self.load_module(nsfile)
                param = {"oldname": osfile, "newname": nsfile}
                op = 'rename'
                ifigure.events.SendFileChangedEvent(self, operation=op,
                                                    param=param)

#           ifigure.events.SendChangedEvent(self)

    def get_filepath(self):
        try:
            pn = self._obj._m_co.pathname
            mn = self._obj._m_co.modename
            return self.path2fullpath(mn, pn)
        except:
            import traceback
            print(traceback.format_exc())
            return ''

    def set_contents(self, *args):
        if hasattr(self._obj._m_co, 'set_contents'):
            m = self._obj._m_co.set_contents
            return m(self, *args)
        else:
            return py_code.PyCode.set_contents(self, *args)
            



#
#  Name   :primitive_widgets
#
#          this module provide a collection of widgets
#
#          there are two groups of widget defined in
#          this file
#
#          1) primitive widgets
#             this is a collection of generic widgets
#             those widgets should hava
#                set_value/get_value procedure
#                self.events : list of events to be returned
#             also, event must be skipped.
#
#          2) widgets for ifigure
#             these are made by combining the above primitives.
#
#             SetCanvasValue/SetEditorValue needs to be implemented.
#

#
#
#  Author :
#         Syun'ichi Shiraiwa
#  E-mail :
#         shiraiwa@psfc.mit.edu
#
#
#  History :
#
# *******************************************
#     Copyright(c) 2012- PSFC, MIT
# *******************************************

from ifigure.widgets.custom_double_slider import CustomDoubleSlider, EVT_CDS_CHANGED
import wx
import random
import os
import ifigure
import ifigure.utils.cbook as cbook
#from ifigure.widgets.base_widget import base_widget
from matplotlib.artist import getp as mpl_getp
from matplotlib.artist import setp as mpl_setp
from ifigure.utils.edit_list import EditListEvent, EditorChanged

dirname = os.path.dirname(ifigure.__file__)


class primitive_widgets(wx.Panel):
    def __init__(self, parent):
        super(primitive_widgets, self).__init__(parent)

        from ifigure.utils.wx3to4 import FlexGridSizer
        sizer = FlexGridSizer(1, 2)

        self.SetSizer(sizer)
        self.Controls = []
        self.events = None
### list events needs to be handled ###
        self.event_obj = None

    def get_control_command(self):
        #        obj=self.event_obj.GetEventObject()
        for ctl in self.Controls:
            if ctl["obj"] is self.event_obj:
                return ctl

    def check_if_need2expand(self):
        sizer = self.GetSizer()
        size = sizer.CalcRowsCols()
        r = sizer.GetRows()

        if size[0] == r:
            #          print 'expanding'
            sizer.SetRows(r+1)

    def add_bitmap_buttons(self, title, ftitle, names, pname):

        topsizer = self.GetSizer()

        gridsizer = wx.GridSizer(10, 4)

        st1 = wx.StaticText(self, label=title)
        topsizer.Add(st1, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 1)

        from ifigure.ifigure_config import icondir
        for name in names:
            imageFile = os.path.join(icondir, 'image',
                                     ftitle+'_'+name+'.png')
            btn = wx.BitmapButton(self, bitmap=wx.Bitmap(imageFile))
            gridsizer.Add(btn, 0, wx.ALL, 0)
            self.Bind(wx.EVT_BUTTON, self.onEdit, btn)
            self.Controls.append(
                {"obj": btn, "property": pname, "value": name})
        topsizer.Add(gridsizer, 0, wx.ALL, 1)

    def add_sample(self):
        colors = ["red", "blue", "gray", "yellow", "green"]
        self.SetBackgroundColour(random.choice(colors))

        btn = wx.Button(self, label="Press Me")
        sizer = self.GetSizer()
        sizer.Add(btn, 0, wx.ALL, 5)

    def add_text_label(self, parent=None, label="", style=wx.TE_PROCESS_ENTER):
        topsizer = parent.GetSizer()
        st1 = wx.StaticText(parent, label=label)
        ct1 = wx.TextCtrl(parent, value="", style=style)
        topsizer.Add(st1, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 1)
        topsizer.Add(ct1, 1, wx.ALL | wx.ALIGN_CENTER_VERTICAL | wx.EXPAND, 1)
        return ct1

    def onEdit(self, event):

        event.SetEventObject(self)
        event.Skip()
        return
        self.event_obj = event.GetEventObject()
        evt = EditListEvent(EditorChanged, wx.ID_ANY)
        evt.SetEventObject(self)
        evt.elp = self
        evt.widget_idx = 0
        handler = self.GetParent()
        handler.ProcessEvent(evt)

    def set_value(self, value):
        pass

    def get_value(self):
        return None


class two_sliders_widget(wx.Panel):
    def __init__(self, parent, minV=0., maxV=1., val1=0.5, val2=0.5, res=0.01, position=(10, 10), size=(100, 20)):
        super(two_sliders_widget, self).__init__(parent)

        self.minV = minV
        self.maxV = maxV
        self.res = res
        self.datamax = (self.maxV-self.minV)/self.res
        self.s1 = wx.Slider(self, wx.ID_ANY, self._val2data(val1), 0,
                            self.datamax, position, (120, -1))
        self.s2 = wx.Slider(self, wx.ID_ANY, self._val2data(val2), 0,
                            self.datamax, position, (120, -1))

        sizer = wx.BoxSizer(wx.VERTICAL)
#        sizer2=wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self.s1, 1, wx.EXPAND | wx.ALL, 2)
        sizer.Add(self.s2, 1, wx.EXPAND | wx.ALL, 2)
#        sizer.Add(sizer2, 1, wx.EXPAND, 0)
        self.SetSizer(sizer)

        self.Bind(wx.EVT_SLIDER, self._sliderUpdate, self.s1)
        self.Bind(wx.EVT_SLIDER, self._sliderUpdate, self.s2)
        self.events = wx.EVT_SLIDER
        self._val = [val1, val2]

    def set_value(self, value):
        self.s1.SetValue(self._val2data(value[0]))
        self.s2.SetValue(self._val2data(value[1]))

    def get_value(self):
        return [self._data2val(self.s1.GetValue()),
                self._data2val(self.s2.GetValue())]

    def _val2data(self, x):
        return (x-self.minV)/(self.maxV-self.minV)*self.datamax

    def _data2val(self, data):
        return (self.maxV-self.minV)*data/self.datamax+self.minV

    def _sliderUpdate(self, event):
        sl = event.GetEventObject()
        val = self.get_value()
        if val[0] > val[1]:
            if sl is self.s1:
                self.s2.SetValue(self._val2data(val[0])+1)
            else:
                self.s1.SetValue(self._val2data(val[1])-1)

        do_skip = False
        if sl is self.s1:
            if self._val[0] == val[0]:
                do_skip = True
        else:
            if self._val[1] == val[1]:
                do_skip = True
        self._val = val

        if do_skip:
            event.SetEventObject(self)
            event.Skip()

#
#   compound widget specifically designed for ifigure
#   these widgets has its own SetEditorValue/SetCanvas
#   Value/onEdit implementation
#


#
#    Margin widget
#


class margin_widget(wx.Panel):

    def __init__(self, parent, id=wx.ID_ANY, *args, **kargs):
        super(margin_widget, self).__init__(parent, id, *args, **kargs)
        self.use_def = wx.CheckBox(self, -1, 'Use Global Setting',
                                   (10, 10))

        st1 = wx.StaticText(self, label="   X  ")
        self.h_slider = CustomDoubleSlider(self, wx.ID_ANY)
        st2 = wx.StaticText(self, label="   Y  ")
        self.v_slider = CustomDoubleSlider(self, wx.ID_ANY)

        big_sizer = wx.BoxSizer(wx.VERTICAL)
        big_sizer2 = wx.BoxSizer(wx.HORIZONTAL)

        from ifigure.utils.wx3to4 import FlexGridSizer
        sizer = FlexGridSizer(2, 2)

        sizer.AddGrowableCol(1)
        sizer.Add(st1, 0, wx.ALIGN_CENTER_VERTICAL, 3)
        sizer.Add(self.h_slider, 1, wx.EXPAND |
                  wx.ALIGN_CENTER_VERTICAL | wx.ALL, 3)
        sizer.Add(st2, 0, wx.ALIGN_CENTER_VERTICAL, 3)
        sizer.Add(self.v_slider, 1, wx.EXPAND |
                  wx.ALIGN_CENTER_VERTICAL | wx.ALL, 3)
        big_sizer.Add(self.use_def, 0)
        big_sizer.Add(sizer, 1, wx.EXPAND)
        big_sizer2.Add(big_sizer, 1, wx.EXPAND)
        self.SetSizer(big_sizer2, wx.EXPAND)

        self.Bind(EVT_CDS_CHANGED, self.onEvent)
#        self.Bind(wx.EVT_SCROLL_CHANGED, self.onEvent)
        self.Bind(wx.EVT_CHECKBOX, self.onEvent)

    def onEvent(self, event):
        if event.GetEventObject() is self.use_def:
            self.SetCanvasValue2()
            return
        self.SetCanvasValue()
        return

    def SetEditorValue(self, use_def, m):
        self.use_def.SetValue(use_def)
        self.h_slider.SetValue([m[0], 1.-m[1]])
        self.v_slider.SetValue([m[2], 1.-m[3]])

    def SetCanvasValue2(self, artist=None):
        a = self.h_slider.GetValue()
        b = self.v_slider.GetValue()
        c = self.use_def.IsChecked()

        se = self.GetParent().GetParent().GetParent()
        pe = se.GetParent()
        canvas = pe.get_canvas()

        f_page = (canvas.get_figure()).figobj
        idx = se.area.index(se.area_hit)
        f_axes = f_page.get_child(idx)
        if c:
            f_axes.setp("use_def_margin", True)
        else:
            f_axes.setp("use_def_margin", False)

        f_page.realize()
        canvas.draw()
        self.GetParent().GetParent().GetParent().SetEditorValue()

    def SetCanvasValue(self, artist=None):
        a = self.h_slider.GetValue()
        b = self.v_slider.GetValue()
        c = self.use_def.IsChecked()

        se = self.GetParent().GetParent().GetParent()
        pe = se.GetParent()
        canvas = pe.get_canvas()

        f_page = (canvas.get_figure()).figobj
        idx = se.area.index(se.area_hit)
        f_axes = f_page.get_child(idx)
        if c:
            f_page.setp("def_margin", [a[0], 1.-a[1], b[0], 1.-b[1]])
            f_page.reset_axesbmp_update()
        else:
            f_axes.setp("margin", [a[0], 1.-a[1], b[0], 1.-b[1]])
            f_axes.set_bmp_update(False)

        f_page.realize()
        canvas.draw()
        self.GetParent().GetParent().GetParent().SetEditorValue()


class marginp_widget(wx.Panel):

    def __init__(self, parent, id=wx.ID_ANY, *args, **kargs):
        super(marginp_widget, self).__init__(parent, id, *args, **kargs)

#        self.use_def= wx.CheckBox(self, -1, 'Use Global Setting',
#                                  (10, 10))

        st1 = wx.StaticText(self, label="   X  ")
        self.h_slider = CustomDoubleSlider(self, wx.ID_ANY)
        st2 = wx.StaticText(self, label="   Y  ")
        self.v_slider = CustomDoubleSlider(self, wx.ID_ANY)

        big_sizer = wx.BoxSizer(wx.VERTICAL)
        big_sizer2 = wx.BoxSizer(wx.HORIZONTAL)

        from ifigure.utils.wx3to4 import FlexGridSizer
        sizer = FlexGridSizer(2, 2)
        sizer.AddGrowableCol(1)
        sizer.Add(st1, 0, wx.ALIGN_CENTER_VERTICAL, 3)
        sizer.Add(self.h_slider, 1, wx.EXPAND |
                  wx.ALIGN_CENTER_VERTICAL | wx.ALL, 3)
        sizer.Add(st2, 0, wx.ALIGN_CENTER_VERTICAL, 3)
        sizer.Add(self.v_slider, 1, wx.EXPAND |
                  wx.ALIGN_CENTER_VERTICAL | wx.ALL, 3)
        big_sizer.Add(sizer, 1, wx.EXPAND)
        big_sizer2.Add(big_sizer, 1, wx.EXPAND)
        self.SetSizer(big_sizer2, wx.EXPAND)

        self.Bind(EVT_CDS_CHANGED, self.onEvent)
#        self.Bind(wx.EVT_SCROLL_CHANGED, self.onEvent)

    def onEvent(self, event):
        self.SetCanvasValue()

    def SetEditorValue(self, m):
        self.h_slider.SetValue([m[0], 1.-m[1]])
        self.v_slider.SetValue([m[2], 1.-m[3]])

    def SetCanvasValue(self, artist=None):
        a = self.h_slider.GetValue()
        b = self.v_slider.GetValue()

        se = self.GetParent().GetParent().GetParent()
        pe = se.GetParent()
        canvas = pe.get_canvas()

        f_page = (canvas.get_figure()).figobj
        f_page._page_margin = [a[0], 1.-a[1], b[0], 1.-b[1]]
        for f_axes in f_page.walk_axes():
            f_axes.set_bmp_update(False)
        f_page.realize()
        canvas.draw()
        self.GetParent().GetParent().GetParent().SetEditorValue()



from __future__ import print_function
# module to run script with __main__ condition

import numpy as np


def EvalNode(___instring___):
    print(___instring___)
    ___instring___
    return eval(___instring___)



from __future__ import print_function

import wx
import tarfile
import os
import gzip
import tempfile
import time

import traceback
from ifigure.widgets.dialog import showtraceback, message

from threading import Lock
local_lc = Lock()

from multiprocessing import Process, Lock

def do_zip(filename, tarname,  d):
    # make .gz file
    #lc.acquire()
    with open(tarname, 'rb') as f_in:
         with gzip.open(filename, 'wb') as f_out:
             f_out.writelines(f_in)
    os.remove(tarname)
    #lc.release()

class MPTarzip(object):
    worker = None
    #lc = Lock()

    def Run(self, filename, d, odir):
        if not self.isReady():
            return False
        try:
            os.getcwd()
        except FileNotFoundError:
            os.chdir(os.path.expanduser("~"))

        print("starting tar....(save)")

        try:
            tarname = self.make_tar(filename, d)
        except PermissionError:
            message(wx.GetApp().TopWindow,
                    'Permission error', 'Error', 0)
            return False
        except:
            showtraceback(wx.GetApp().TopWindow, txt=traceback.format_exc())
            return False

        print("starting tar.gz....(save)")
        MPTarzip.worker = Process(target=do_zip, args=(filename, tarname, d))

        try:
            MPTarzip.worker.start()
        except:
            showtraceback(wx.GetApp().TopWindow, txt=traceback.format_exc())
            return False

        self.odir = odir
        self.d = d
        wx.CallLater(100, self.CheckFinished)

        return True

    def make_tar(self, filename, d):
        # make tar.gz file
        #        lc.acquire()
        fid = tempfile.NamedTemporaryFile('w+b',
                                          dir=os.path.dirname(filename),
                                          delete=False)
        #fid = open(filename+'.tar', 'w')
        tfid = tarfile.open(mode='w:', fileobj=fid)
        basename = os.path.basename(d)
        for item in os.listdir(d):
            # not to save '.trash' directory which
            # is used for temporary data
            if item != '.trash':
                # print(item)
                tfid.add(os.path.join(d, item),
                         arcname=os.path.join(basename, item))
        tfid.close()
        fid.close()
#        lc.release()
        return fid.name

    def CheckFinished(self):
        if MPTarzip.worker.is_alive():
            top = wx.GetApp().TopWindow
            if top is not None:
                top.set_window_title()
            wx.CallLater(2000, self.CheckFinished)
        else:
            if self.odir != self.d:
                app = wx.GetApp().TopWindow
                app.proj._delete_tempdir(self.odir)
            top = wx.GetApp().TopWindow
            if top is not None:
                top.set_window_title()
            MPTarzip.worker = None
            print("...finished (save)")
            local_lc.release()

    def isReady(self):
        if MPTarzip.worker is None:
            return True
        if MPTarzip.worker.is_alive():
            return False
        return True



from __future__ import print_function
#
#  description
#     make_release generate a release version directory
#     if runs hg log and hg summary
#

import shutil
import os
import time
import subprocess
import traceback
import ifigure

check = ['ifigure_app', 'startup', 'add_on']


def header_text(f):
    txt = ('#  Title:\t' + f,
           '#',
           '#  Author:\tSyun\'ichi Shiraiwa',
           '#',
           '#  E-mail:\tshiraiwa@psfc.mit.edu',
           '#',
           '#  Notice:\t Beta version restriction.',
           '#         \t Do not copy this version. Although piScope will',
           '#         \t distributed under GPL, the distributing this version',
           '#         \t and any derived work is not allowd',
           '#*******************************************',
           '#     Copyright(c) 2012-14 S. Shiraiwa      ',
           '#*******************************************',
           '',)
    return '\n'.join(txt)


def make_header(release='piscope_release'):
    source = os.path.dirname(os.path.dirname(
        os.path.dirname(ifigure.__file__)))
    root = os.path.dirname(os.path.dirname(
        os.path.dirname(os.path.dirname(ifigure.__file__))))
    release = os.path.join(root, release)

    c = 0
    for dirpath, dirname, filenames in os.walk(release):
        for f in filenames:
            path = os.path.join(dirpath, f)
            if path.endswith('~'):
                os.remove(path)
                continue

            if path.endswith('.py'):
                lines = open(path, 'r').readlines()
                if len(lines) == 0:
                    continue
                if lines[0].startswith('#!'):
                    continue
                for l in lines:
                    if l.upper().find('COPYRIGHT(C)') != -1:
                        print(('skipping ', path))
                        break
                else:
                    #                     print 'processing ', path
                    fid = open(path, 'w')
                    fid.write(header_text(f))
                    for l in lines:
                        fid.write(l)
                    fid.close()
                    c = c + 1
    print(('total ', str(c) + ' files are processed'))
#            if c > 3: return


def check_print():
    # check if print statement is used
    source = os.path.dirname(os.path.dirname(
        os.path.dirname(ifigure.__file__)))

    c = 0
    for dirpath, dirname, filenames in os.walk(source):
        for f in filenames:
            path = os.path.join(dirpath, f)
            if path.endswith('~'):
                os.remove(path)
                continue

            if path.endswith('.py'):
                lines = open(path, 'r').readlines()
                if len(lines) == 0:
                    continue
                for l in lines:
                    if l.upper().find('PRINT ') != -1:
                        if l.find('#') != -1:
                            if l.find('#') < l.upper().find('PRINT '):
                                continue
                        print((path, l))
                        c = c + 1
    print(('total ', str(c) + ' PRINT statement detected'))
#            if c > 3: return


def make_release(release='piscope_release', version='beta'):
    '''
    make_release(release = 'piscope_release', version = 'beta')
    '''
    source = os.path.dirname(os.path.dirname(
        os.path.dirname(ifigure.__file__)))

    pwd = os.getcwd()
    try:
        os.chdir(source)
        log = subprocess.check_output(['git', 'log'])
        fid = open(os.path.join(source, 'git_log'), 'w')
        fid.write(log)
        fid.close()
        #log = subprocess.check_output(['hg', 'summary'])
        #fid = open(os.path.join(source, 'hg_summary'), 'w')
        # fid.write(log)
        # fid.close()
        root = os.path.dirname(os.path.dirname(
            os.path.dirname(os.path.dirname(ifigure.__file__))))
        release = os.path.join(root, release)
        version_file = os.path.join(release, 'python', 'ifigure', 'version.py')

        if os.path.exists(release):
            try:
                shutil.rmtree(release)
            except:
                pass
        shutil.copytree(source, release)

        for dirpath, dirname, filenames in os.walk(release):
            for f in filenames:

                path = os.path.join(dirpath, f)

                if path.endswith('~'):
                    os.remove(path)
                    continue
    #            if any([path.find(x) != -1 for x in check]): continue
    #            if path.endswith('.py'):
    #                 os.remove(path)
                if f.startswith('._'):
                    #                 print path
                    os.remove(path)
                    continue
                if f.startswith('.DS_Store'):
                    os.remove(path)
                    continue
                if f.startswith('.nfs'):
                    print(('removing ', path))
                    os.remove(path)
                    continue

    #            print 'processed ', path
        fid = open(version_file, 'w')
        fid.write('ifig_version = "' + time.strftime('%Y %B %d') +
                  '_' + version + ' with OpenGL backend"\n')
        fid.close()

        make_header(release=release)
    except:
        traceback.print_exc()
    finally:
        os.chdir(pwd)
